@@ -1,6 +1,6 @@
-static const char *TagName   = "$Name:  $";
+static const char *TagName   = "$Name: DataBase-Release_2_9 $";
 static const char *HttpServer= "http://www.esrf.fr/computing/cs/tango/tango_doc/ds_doc/";
-static const char *RcsId = "$Header: /cvsroot/tango-cs/tango/cppserver/dbase/DataBaseClass.cpp,v 2.29 2005/10/05 07:35:54 pascal_verdier Exp $";
+static const char *RcsId = "$Header: /cvsroot/tango-cs/tango/cppserver/dbase/DataBaseClass.cpp,v 2.32 2006/09/28 11:18:20 pascal_verdier Exp $";
 //+=============================================================================
 //
 // file :        DataBaseClass.cpp
@@ -12,11 +12,20 @@ static const char *RcsId = "$Header: /cvsroot/tango-cs/tango/cppserver/dbase/Dat
 //
 // project :     TANGO Device Server
 //
-// author(s) :   A.Gotz
+// author(s) :   A.Gotz, JL Pons, P Verdier
 //
 // $Version$
 //
 // $Log: DataBaseClass.cpp,v $
+// Revision 2.32  2006/09/28 11:18:20  pascal_verdier
+// DbGetClassForDevice and DbGetClassInheritanceForDevice commands added.
+//
+// Revision 2.31  2006/08/23 08:57:34  pascal_verdier
+// Header changed.
+//
+// Revision 2.30  2006/06/22 15:25:32  jlpons
+// Added history commands
+//
 // Revision 2.29  2005/10/05 07:35:54  pascal_verdier
 // Bug fixed in Timing_Info attribute read.
 // ResetTimingValues command added.
@@ -173,6 +182,176 @@ static const char *RcsId = "$Header: /cvsroot/tango-cs/tango/cppserver/dbase/Dat
 
 namespace DataBase_ns {
 
+//+----------------------------------------------------------------------------
+//
+// method : 		DbGetClassInheritanceForDeviceClass::execute()
+// 
+// description : 	method to trigger the execution of the command.
+//                PLEASE DO NOT MODIFY this method core without pogo   
+//
+// in : - device : The device on which the command must be excuted
+//		- in_any : The command input data
+//
+// returns : The command output data (packed in the Any object)
+//
+//-----------------------------------------------------------------------------
+CORBA::Any *DbGetClassInheritanceForDeviceClass::execute(Tango::DeviceImpl *device,const CORBA::Any &in_any)
+{
+
+	cout2 << "DbGetClassInheritanceForDeviceClass::execute(): arrived" << endl;
+
+	Tango::DevString	argin;
+	extract(in_any, argin);
+
+	return insert((static_cast<DataBase *>(device))->db_get_class_inheritance_for_device(argin));
+}
+
+//+----------------------------------------------------------------------------
+//
+// method : 		DbGetClassForDeviceClass::execute()
+// 
+// description : 	method to trigger the execution of the command.
+//                PLEASE DO NOT MODIFY this method core without pogo   
+//
+// in : - device : The device on which the command must be excuted
+//		- in_any : The command input data
+//
+// returns : The command output data (packed in the Any object)
+//
+//-----------------------------------------------------------------------------
+CORBA::Any *DbGetClassForDeviceClass::execute(Tango::DeviceImpl *device,const CORBA::Any &in_any)
+{
+
+	cout2 << "DbGetClassForDeviceClass::execute(): arrived" << endl;
+
+	Tango::DevString	argin;
+	extract(in_any, argin);
+
+	return insert((static_cast<DataBase *>(device))->db_get_class_for_device(argin));
+}
+
+
+//+----------------------------------------------------------------------------
+//
+// method : 		DbGetClassAttributePropertyHistCmd::execute()
+// 
+// description : 	method to trigger the execution of the command.
+//                PLEASE DO NOT MODIFY this method core without pogo   
+//
+// in : - device : The device on which the command must be excuted
+//		- in_any : The command input data
+//
+// returns : The command output data (packed in the Any object)
+//
+//-----------------------------------------------------------------------------
+CORBA::Any *DbGetClassAttributePropertyHistCmd::execute(Tango::DeviceImpl *device,const CORBA::Any &in_any)
+{
+
+	cout2 << "DbGetClassAttributePropertyHistCmd::execute(): arrived" << endl;
+
+	const Tango::DevVarStringArray	*argin;
+	extract(in_any, argin);
+
+	return insert((static_cast<DataBase *>(device))->db_get_class_attribute_property_hist(argin));
+}
+
+//+----------------------------------------------------------------------------
+//
+// method : 		DbGetDeviceAttributePropertyHistCmd::execute()
+// 
+// description : 	method to trigger the execution of the command.
+//                PLEASE DO NOT MODIFY this method core without pogo   
+//
+// in : - device : The device on which the command must be excuted
+//		- in_any : The command input data
+//
+// returns : The command output data (packed in the Any object)
+//
+//-----------------------------------------------------------------------------
+CORBA::Any *DbGetDeviceAttributePropertyHistCmd::execute(Tango::DeviceImpl *device,const CORBA::Any &in_any)
+{
+
+	cout2 << "DbGetDeviceAttributePropertyHistCmd::execute(): arrived" << endl;
+
+	const Tango::DevVarStringArray	*argin;
+	extract(in_any, argin);
+
+	return insert((static_cast<DataBase *>(device))->db_get_device_attribute_property_hist(argin));
+}
+
+//+----------------------------------------------------------------------------
+//
+// method : 		DbGetPropertyHistClass::execute()
+// 
+// description : 	method to trigger the execution of the command.
+//                PLEASE DO NOT MODIFY this method core without pogo   
+//
+// in : - device : The device on which the command must be excuted
+//		- in_any : The command input data
+//
+// returns : The command output data (packed in the Any object)
+//
+//-----------------------------------------------------------------------------
+CORBA::Any *DbGetPropertyHistClass::execute(Tango::DeviceImpl *device,const CORBA::Any &in_any)
+{
+
+	cout2 << "DbGetPropertyHistClass::execute(): arrived" << endl;
+
+	const Tango::DevVarStringArray	*argin;
+	extract(in_any, argin);
+
+	return insert((static_cast<DataBase *>(device))->db_get_property_hist(argin));
+}
+
+//+----------------------------------------------------------------------------
+//
+// method : 		DbGetClassPropertyHistClass::execute()
+// 
+// description : 	method to trigger the execution of the command.
+//                PLEASE DO NOT MODIFY this method core without pogo   
+//
+// in : - device : The device on which the command must be excuted
+//		- in_any : The command input data
+//
+// returns : The command output data (packed in the Any object)
+//
+//-----------------------------------------------------------------------------
+CORBA::Any *DbGetClassPropertyHistClass::execute(Tango::DeviceImpl *device,const CORBA::Any &in_any)
+{
+
+	cout2 << "DbGetClassPropertyHistClass::execute(): arrived" << endl;
+
+	const Tango::DevVarStringArray	*argin;
+	extract(in_any, argin);
+
+	return insert((static_cast<DataBase *>(device))->db_get_class_property_hist(argin));
+}
+
+
+
+//+----------------------------------------------------------------------------
+//
+// method : 		DbGetDevicePropertyHistClass::execute()
+// 
+// description : 	method to trigger the execution of the command.
+//                PLEASE DO NOT MODIFY this method core without pogo   
+//
+// in : - device : The device on which the command must be excuted
+//		- in_any : The command input data
+//
+// returns : The command output data (packed in the Any object)
+//
+//-----------------------------------------------------------------------------
+CORBA::Any *DbGetDevicePropertyHistClass::execute(Tango::DeviceImpl *device,const CORBA::Any &in_any)
+{
+
+	cout2 << "DbGetDevicePropertyHistClass::execute(): arrived" << endl;
+
+	const Tango::DevVarStringArray	*argin;
+	extract(in_any, argin);
+
+	return insert((static_cast<DataBase *>(device))->db_get_device_property_hist(argin));
+}
 
 //+----------------------------------------------------------------------------
 //
@@ -1833,6 +2012,7 @@ DataBaseClass::DataBaseClass(string &s):Tango::DeviceClass(s)
 {
 
 	cout2 << "Entering DataBaseClass constructor" << endl;
+	get_class_property();
 	set_default_property();
 	write_class_property();
 	
@@ -2008,6 +2188,21 @@ void DataBaseClass::command_factory()
 		"",
 		"",
 		Tango::OPERATOR));
+	command_list.push_back(new DbGetClassAttributePropertyHistCmd("DbGetClassAttributePropertyHist",
+		Tango::DEVVAR_STRINGARRAY, Tango::DEVVAR_STRINGARRAY,
+		"",
+		"",
+		Tango::OPERATOR));
+	command_list.push_back(new DbGetClassForDeviceClass("DbGetClassForDevice",
+		Tango::DEV_STRING, Tango::DEV_STRING,
+		"Device name",
+		"Class off the specified device",
+		Tango::OPERATOR));
+	command_list.push_back(new DbGetClassInheritanceForDeviceClass("DbGetClassInheritanceForDevice",
+		Tango::DEV_STRING, Tango::DEVVAR_STRINGARRAY,
+		"Device name",
+		"Classes off the specified device.\n[0] - is the class of the device.\n[1] - is the class from the device class is inherited.\n........and so on",
+		Tango::OPERATOR));
 	command_list.push_back(new DbGetClassList("DbGetClassList",
 		Tango::DEV_STRING, Tango::DEVVAR_STRINGARRAY,
 		"",
@@ -2018,6 +2213,11 @@ void DataBaseClass::command_factory()
 		"",
 		"",
 		Tango::OPERATOR));
+	command_list.push_back(new DbGetClassPropertyHistClass("DbGetClassPropertyHist",
+		Tango::DEVVAR_STRINGARRAY, Tango::DEVVAR_STRINGARRAY,
+		"",
+		"",
+		Tango::OPERATOR));
 	command_list.push_back(new DbGetClassPropertyList("DbGetClassPropertyList",
 		Tango::DEV_STRING, Tango::DEVVAR_STRINGARRAY,
 		"",
@@ -2048,6 +2248,11 @@ void DataBaseClass::command_factory()
 		"",
 		"",
 		Tango::OPERATOR));
+	command_list.push_back(new DbGetDeviceAttributePropertyHistCmd("DbGetDeviceAttributePropertyHist",
+		Tango::DEVVAR_STRINGARRAY, Tango::DEVVAR_STRINGARRAY,
+		"",
+		"",
+		Tango::OPERATOR));
 	command_list.push_back(new DbGetDeviceClassList("DbGetDeviceClassList",
 		Tango::DEV_STRING, Tango::DEVVAR_STRINGARRAY,
 		"",
@@ -2088,6 +2293,11 @@ void DataBaseClass::command_factory()
 		"",
 		"",
 		Tango::OPERATOR));
+	command_list.push_back(new DbGetDevicePropertyHistClass("DbGetDevicePropertyHist",
+		Tango::DEVVAR_STRINGARRAY, Tango::DEVVAR_STRINGARRAY,
+		"",
+		"",
+		Tango::OPERATOR));
 	command_list.push_back(new DbGetDevicePropertyList("DbGetDevicePropertyList",
 		Tango::DEVVAR_STRINGARRAY, Tango::DEVVAR_STRINGARRAY,
 		"",
@@ -2133,6 +2343,11 @@ void DataBaseClass::command_factory()
 		"",
 		"",
 		Tango::OPERATOR));
+	command_list.push_back(new DbGetPropertyHistClass("DbGetPropertyHist",
+		Tango::DEVVAR_STRINGARRAY, Tango::DEVVAR_STRINGARRAY,
+		"",
+		"",
+		Tango::OPERATOR));
 	command_list.push_back(new DbGetPropertyList("DbGetPropertyList",
 		Tango::DEVVAR_STRINGARRAY, Tango::DEVVAR_STRINGARRAY,
 		"",
@@ -2293,7 +2508,6 @@ Tango::DbDatum DataBaseClass::get_default_class_property(string &prop_name)
 	//	if not found, return  an empty DbDatum
 	return Tango::DbDatum(prop_name);
 }
-
 //+----------------------------------------------------------------------------
 //
 // method : 		DataBaseClass::device_factory
@@ -2306,26 +2520,13 @@ Tango::DbDatum DataBaseClass::get_default_class_property(string &prop_name)
 //-----------------------------------------------------------------------------
 void DataBaseClass::device_factory(const Tango::DevVarStringArray *devlist_ptr)
 {
-
-	//	Create all devices.(Automatic code generation)
-	//-------------------------------------------------------------
-
-	//	End of Automatic code generation
-	//-------------------------------------------------------------
-
-
-/**
- *	This  method is not standard for DataBaseClass.cpp file.
- *
- *	Remove the POGO automatic generated code.
- */
 	cout << "DataBaseClass::DataBaseClass(): create database device " << DataBase::db_name.c_str() << endl;
 	device_list.push_back(new DataBase(this, DataBase::db_name.c_str(),
                                     	"TANGO database device server"));
 
 	export_device(device_list[0],"database");
-
 }
+
 //+----------------------------------------------------------------------------
 //	Method: DataBaseClass::attribute_factory(vector<Tango::Attr *> &att_list)
 //-----------------------------------------------------------------------------
@@ -2357,6 +2558,35 @@ void DataBaseClass::attribute_factory(vector<Tango::Attr *> &att_list)
 
 }
 
+//+----------------------------------------------------------------------------
+//
+// method : 		DataBaseClass::get_class_property()
+// 
+// description : 	Read the class properties from database.
+//
+//-----------------------------------------------------------------------------
+void DataBaseClass::get_class_property()
+{
+	//	Initialize your default values here (if not done with  POGO).
+	//------------------------------------------------------------------
+
+	//	Read class properties from database.(Automatic code generation)
+	//------------------------------------------------------------------
+	if (Tango::Util::instance()->_UseDb==false)
+		return;
+
+	//	Call database and extract values
+	//--------------------------------------------
+	get_db_class()->get_property(cl_prop);
+	Tango::DbDatum	def_prop;
+	int	i = -1;
+
+
+	//	End of Automatic code generation
+	//------------------------------------------------------------------
+
+}
+
 //+----------------------------------------------------------------------------
 //
 // method : 	DataBaseClass::set_default_property
