@@ -1,4 +1,4 @@
-static const char *RcsId = "$Header: /cvsroot/tango-cs/tango/cppserver/starter/Starter.cpp,v 3.17 2005/10/04 13:02:52 pascal_verdier Exp $";
+static const char *RcsId = "$Header: /cvsroot/tango-cs/tango/cppserver/starter/Starter.cpp,v 3.26 2006/06/13 19:38:15 pascal_verdier Exp $";
 //+=============================================================================
 //
 // file :         Starter.cpp
@@ -13,9 +13,38 @@ static const char *RcsId = "$Header: /cvsroot/tango-cs/tango/cppserver/starter/S
 //
 // $Author: pascal_verdier $
 //
-// $Revision: 3.17 $
+// $Revision: 3.26 $
 //
 // $Log: Starter.cpp,v $
+// Revision 3.26  2006/06/13 19:38:15  pascal_verdier
+// Minor changes.
+//
+// Revision 3.25  2006/06/06 12:01:26  pascal_verdier
+// Bug in log file fixed.
+//
+// Revision 3.24  2006/06/05 07:20:36  pascal_verdier
+// Server startup is now delayed (with timeout) in a startup level.
+// New state MOVING added.
+// At startup, starter loop until properties have been read.
+//
+// Revision 3.23  2006/05/15 10:52:37  pascal_verdier
+// Remove a ifndef WIN32 (?)
+//
+// Revision 3.22  2006/04/24 07:06:27  pascal_verdier
+// A thread is started for each level when at servers startup.
+//
+// Revision 3.21  2006/02/10 13:28:35  pascal_verdier
+// *** empty log message ***
+//
+// Revision 3.20  2006/02/09 11:59:18  pascal_verdier
+// A ping thread is now started for each server.
+//
+// Revision 3.19  2006/02/08 07:13:55  pascal_verdier
+// Minor changes.
+//
+// Revision 3.18  2006/01/13 15:18:53  pascal_verdier
+// Bug on notifd startup fixed.
+//
 // Revision 3.17  2005/10/04 13:02:52  pascal_verdier
 // *** empty log message ***
 //
@@ -128,7 +157,7 @@ static const char *RcsId = "$Header: /cvsroot/tango-cs/tango/cppserver/starter/S
 
 //===================================================================
 //
-//	The folowing table gives the correspondance
+//	The following table gives the correspondance
 //	between commands and method's name.
 //
 //  Command's name        |  Method's name
@@ -143,7 +172,6 @@ static const char *RcsId = "$Header: /cvsroot/tango-cs/tango/cppserver/starter/S
 //  DevStop               |  dev_stop()
 //  DevReadLog            |  dev_read_log()
 //  UpdateServersInfo     |  update_servers_info()
-//  StartPollingThread    |  start_polling_thread()
 //  NotifyDaemonState     |  notify_daemon_state()
 //
 //===================================================================
@@ -197,52 +225,20 @@ namespace Starter_ns
 {
 //+------------------------------------------------------------------
 /**
- *	Check if server running.
- *	Try to import his device administrator.
- *
- *	@param devname	Device name.
+ *	Class Destructor
  */
 //+------------------------------------------------------------------
-const bool Starter::is_running(ServerControled *server)
+Starter::~Starter()
 {
-#ifdef DisplayTime
-	TimeVal	before, after;
-	GetTime(before);
-#endif
-	bool	retval;
-	try
+	//	Stop ping threads
+	vector<ControledServer>::iterator it;
+	for (it=servers.begin() ; it<servers.end() ; it++)
 	{
-		if (server->dev == NULL)
-		{
-			server->dev = new Tango::DeviceProxy(server->admin_name);
-			server->dev->set_timeout_millis(PING_TIMEOUT);
-		}
-		//cout  << "pinging |" << server->admin_name << "|" << endl;;
-		int t = server->dev->ping();
-		retval = true;
+		it->thread_data->set_stop_thread();
 	}
-	catch(Tango::DevFailed &e)
-	{
-		retval = false;
-		if (server->dev!=NULL)
-		{
-			delete server->dev;
-			server->dev = NULL;
-		}
-		//Tango::Except::print_exception(e);
-	}
-#ifdef DisplayTime
-	GetTime(after);
-	//if (Elapsed(before, after) > 100)
-	string::size_type	pos = server->admin_name.find("Dummy");
-	if (pos != string::npos)
-	cout << "ping " << server->admin_name << "  " << Elapsed(before, after) << " ms" << endl;
-#endif
-	//cout << "retval = " << retval << endl;
-	return retval;
+	ms_sleep(1000);
 }
 
-
 //+----------------------------------------------------------------------------
 //
 // method : 		Starter::Starter(string &s)
@@ -285,8 +281,17 @@ void Starter::init_device()
 
 	// Initialise variables to default values.
 	//--------------------------------------------
-	get_device_property();
-
+	bool	read_properties_done = false;
+	while (!read_properties_done)
+	{
+		try {
+			get_device_property();
+			read_properties_done = true;
+		}
+		catch(Tango::DevFailed &e) {
+			Tango::Except::print_exception(e);
+		}
+	}
 	//	Do it only at startup and not at Init command
 	//----------------------------------------------------
 	if (starting==true)
@@ -299,7 +304,7 @@ void Starter::init_device()
 		//--------------------------------------
 		dbase = new Tango::DeviceProxy(dbname);
 		dbase->set_timeout_millis(500);
-
+		starting_processes = 0;
 
 		//	Get hostname (In case of cluster host could be multiple)
 		//-------------------------------------------------------------
@@ -346,11 +351,7 @@ void Starter::init_device()
 		{
 			cout << "Waiting " << waitForDriverStartup <<
 					" seconds before starting (wait for drivers)." << endl;
-#		ifdef WIN32
-			_sleep(1000*waitForDriverStartup);
-#		else
-			sleep(waitForDriverStartup);
-#		endif
+			ms_sleep(1000*waitForDriverStartup);
 		}
 
 		//	Start notify daemon if not desabled and not already running
@@ -361,11 +362,11 @@ void Starter::init_device()
 				cout << "Checking " << util->notifyd_name << endl;
 				if (util->is_notifyd_alive()!=Tango::ON)
 				{
-					cout << "Starting " << util->notifyd_name << endl;
 					string	name(NOTIFY_DAEMON_SCRIPT);
-					name + "/";
+					name += "/";
 					name += tg->get_host_name();
-						dev_start((char*)name.c_str());
+					cout << "Starting " << name << endl;
+					dev_start((char*)name.c_str());
 				}
 			}
 			catch (...) {}
@@ -383,33 +384,26 @@ void Starter::init_device()
 			}
 			catch(Tango::DevFailed &e) {
 				Tango::Except::print_exception(e);
+			}
 #			ifdef WIN32
 				_sleep(1000);
 #			else
 				sleep(1);
 #			endif
-			}
 		}
+
 		//	And Start servers for all startup levels.
-		//--------------------------------------------
+		//	The interStartupLevelWait value will be managed
+		//		by the start process thread.
+		//---------------------------------------------------
 		int nb_levels =
 			((static_cast<StarterClass *>(get_device_class()))->nbStartupLevels);
 
 		if (startServersAtStartup==true)
 			for (int level=1 ; level<=nb_levels ; level++)
 			{
-				//	Check if wait is needed
-				if (level>1 && interStartupLevelWait>0 &&
-					nb_servers_to_start(level)>0)
-				{
-					//	Wait a bit between startup levels 
-#					ifdef WIN32
-						_sleep(1000*interStartupLevelWait);
-#					else
-						sleep(interStartupLevelWait);
-#					endif
-				}
 				dev_start_all(level);
+				ms_sleep(50);
 			}
 
 		//	Want exception during normal run
@@ -417,17 +411,17 @@ void Starter::init_device()
 
 		//	Set the default state
 		//-------------------------------
-		set_state(Tango::ON);
-		set_status("Tango::ON");
+		set_state(Tango::MOVING);
+		set_status("Tango::MOVING");
 		*attr_HostState_read = get_state();
 
-		shared = NULL;
-		poller = NULL;
-		polling_id = NULL;
-
 		//	Update Loggs
 		WARN_STREAM << "Starter Server Started !" << endl;
 		cout << "Starter Server Started !" << endl;
+
+		//	Start a thread to start polling
+		PollingState	*poller = new PollingState(get_name());
+		poller->start();
 	}
 }
 
@@ -455,6 +449,7 @@ void Starter::get_device_property()
 	dev_prop.push_back(Tango::DbDatum("UseEvents"));
 	dev_prop.push_back(Tango::DbDatum("StartServersAtStartup"));
 	dev_prop.push_back(Tango::DbDatum("InterStartupLevelWait"));
+	dev_prop.push_back(Tango::DbDatum("ServerStartupTimeout"));
 
 	//	Call database and extract values
 	//--------------------------------------------
@@ -509,6 +504,15 @@ void Starter::get_device_property()
 	//	And try to extract InterStartupLevelWait value from database
 	if (dev_prop[i].is_empty()==false)	dev_prop[i]  >>  interStartupLevelWait;
 
+	//	Try to initialize ServerStartupTimeout from class property
+	cl_prop = ds_class->get_class_property(dev_prop[++i].name);
+	if (cl_prop.is_empty()==false)	cl_prop  >>  serverStartupTimeout;
+	//	Try to initialize ServerStartupTimeout from default device value
+	def_prop = ds_class->get_default_device_property(dev_prop[i].name);
+	if (def_prop.is_empty()==false)	def_prop  >>  serverStartupTimeout;
+	//	And try to extract ServerStartupTimeout value from database
+	if (dev_prop[i].is_empty()==false)	dev_prop[i]  >>  serverStartupTimeout;
+
 
 
 	//	End of Automatic code generation
@@ -523,6 +527,7 @@ void Starter::get_device_property()
 	INFO_STREAM << "WaitForDriverStartup = " << waitForDriverStartup << " seconds" << endl;
 	cout << "UseEvents  = " << ((useEvents==false)? "False": "True") << endl;
 	cout << "interStartupLevelWait  = " << interStartupLevelWait << endl;
+	cout << "serverStartupTimeout   = " << serverStartupTimeout << endl;
 
 
 
@@ -549,21 +554,6 @@ void Starter::get_device_property()
 //-----------------------------------------------------------------------------
 void Starter::always_executed_hook()
 {
-	if (polling_id==NULL)
-	{
-		//	Start polling
-		manage_PollingState_startup();
-		//	and store polling thread ID
-		Tango::Util *tg = Tango::Util::instance();
-		polling_id = tg->get_polling_thread_object();
-	}
-	if (shared !=NULL)
-	{
-		//	Do it only if command is not form polling thread
-		omni_thread	*this_thread = omni_thread::self();
-		if (this_thread != polling_id)
-			shared->set_timer();
-	}
 }
 //+----------------------------------------------------------------------------
 //
@@ -607,7 +597,9 @@ void Starter::read_RunningServers(Tango::Attribute &attr)
 	{
 		//	Retreive max_x of spectrum attribute
 		Tango::SpectrumAttr	*specAtt = (Tango::SpectrumAttr	*) &attr;
-		attr_RunningServers_read = new Tango::DevString[specAtt->get_max_x()];
+		//cout << "Running specAtt->get_max_x()  = " << specAtt->get_max_x() << endl;
+		//attr_RunningServers_read = new Tango::DevString[specAtt->get_max_x()];
+		attr_RunningServers_read = new Tango::DevString[NB_SERVERS_MAX];
 	}
 
 	//	Check if servers object initilized
@@ -644,7 +636,9 @@ void Starter::read_StoppedServers(Tango::Attribute &attr)
 	{
 		//	Retreive max_x of spectrum attribute
 		Tango::SpectrumAttr	*specAtt = (Tango::SpectrumAttr	*) &attr;
-		attr_StoppedServers_read = new Tango::DevString[specAtt->get_max_x()];
+		//cout << "Stopped specAtt->get_max_x()  = " << specAtt->get_max_x() << endl;
+		//attr_StoppedServers_read = new Tango::DevString[specAtt->get_max_x()];
+		attr_StoppedServers_read = new Tango::DevString[NB_SERVERS_MAX];
 	}
 
 	//	Check if servers object initilized
@@ -664,6 +658,27 @@ void Starter::read_StoppedServers(Tango::Attribute &attr)
 	attr.set_value(attr_StoppedServers_read, nb);
 }
 
+//+------------------------------------------------------------------
+/**
+ *	Check if server running.
+ *	Try to import his device administrator.
+ *
+ *	@param devname	Device name.
+ */
+//+------------------------------------------------------------------
+const bool Starter::is_running(ControledServer *server)
+{
+	bool	retval;
+	//	Check if thread has updated.
+	time_t	t0 = server->thread_data->get_last_write_time();
+	time_t	dt = time(NULL)-t0;
+	if (dt > PING_TIMEOUT)
+		retval = false;
+	else
+		retval = (server->thread_data->get_state()==Tango::ON);
+	return retval;
+}
+
 //+------------------------------------------------------------------
 /**
  *	method:	Starter::dev_state
@@ -679,6 +694,7 @@ void Starter::read_StoppedServers(Tango::Attribute &attr)
 Tango::DevState Starter::dev_state()
 {
 	INFO_STREAM << "Starter::dev_state(): entering... !" << endl;
+	static int	cnt = 0;
 
 	//	Check if last command is more than readInfoDbPeriod class property
 	int	period =
@@ -703,9 +719,12 @@ Tango::DevState Starter::dev_state()
 	else
 	if (do_update_from_db)
 	{
+		//	do it two times to be sure (I know it is not really nice)
+		cnt++;
+		if ((cnt%2)==0)
+			do_update_from_db = false;
 		//	Has been fired from Dbase
 		util->build_server_ctrl_object(&servers);
-		do_update_from_db = false;
 	}
 	//	Check for notify daemon state if requested
 	//---------------------------------------------
@@ -728,7 +747,7 @@ Tango::DevState Starter::dev_state()
 
 	//	Check hown many servers are running
 	//-----------------------------------------------------------
-	ServerControled		*p_serv;
+	ControledServer		*p_serv;
 	int		nb_running   = 0;
 	int		nb_controled = 0;
 	int		i;
@@ -746,213 +765,37 @@ Tango::DevState Starter::dev_state()
 	}
 
 	//	compare nb running with nb_controlled controled to set state
+	if (starting_processes>0)
+		set_state(Tango::MOVING);
+	else
 	if (nb_running==nb_controled && notifyd_state==Tango::ON)
 		set_state(Tango::ON);
 	else
 		set_state(Tango::ALARM);
 
+	//cout << DeviceImpl::dev_state() << endl;
 //time_t 	t2 = time(NULL);
 //cout << "------------------------------------> " << (t2-t1) << " seconds" << endl;
 
 	return DeviceImpl::dev_state();
 }
 
+
 //+------------------------------------------------------------------
 /**
- *	Add the Polling object.
+ *	Check if a process could be started (file exists, is not running, ...)
  */
 //+------------------------------------------------------------------
-void Starter::add_obj_polling(string type, string name, int upd_period)
+NewProcess *Starter::processCouldStart(char *argin)
 {
-	Tango::Util *tg = Tango::Util::instance();
-	Tango::DServer		*adm_dev = tg->get_dserver_device();
-	Tango::DevVarLongStringArray	*lsa = new Tango::DevVarLongStringArray();
-	lsa->lvalue.length(1);
-	lsa->svalue.length(3);
-	lsa->lvalue[0] = upd_period;
-	lsa->svalue[0] = CORBA::string_dup(get_name().c_str());
-	lsa->svalue[1] = CORBA::string_dup(type.c_str());
-	lsa->svalue[2] = CORBA::string_dup(name.c_str());
-	CORBA::Any	any;
-	any <<= lsa;
-	try {
-		INFO_STREAM <<"AddObjPolling()" << endl;
-		adm_dev->command_inout("AddObjPolling", any);
-	}
-	catch (Tango::DevFailed &e) {
-
-		string	reason(e.errors[0].reason);
-		if (reason=="API_AlreadyPolled")
-		{
-			//	If already polled -> just update period.
-			//----------------------------------------------
-			try {
-				INFO_STREAM <<"UpdObjPollingPeriod(" << upd_period << " ms)" << endl;
-				adm_dev->command_inout("UpdObjPollingPeriod", any);
-			}
-			catch (Tango::DevFailed &e) {
-				Tango::Except::print_exception(e);
-			}
-		}
-		else
-			Tango::Except::print_exception(e);
-	}
-}
-//+------------------------------------------------------------------
-/**
- *	Manage the PollingState object startup
- */
-//+------------------------------------------------------------------
-void Starter::manage_PollingState_startup()
-{
-	//	if poller not yet started.
-	if (poller == NULL)
-	{
-		long	upd_period = 1000 *
-			((static_cast<StarterClass *>(get_device_class()))->readInfoDbPeriod)/2;
-
-		add_obj_polling("command", "State", upd_period);
-
-		//	Create the shared data and Start the state poller thread
-		shared = new SharedData();
-		shared->set_timer();
-		long	timeout =
-			((static_cast<StarterClass *>(get_device_class()))->cmdPollingTimeout);
-		poller = new PollingState(shared, timeout);
-		poller->start();
-	}
-}
-//+------------------------------------------------------------------
-/**
- *	method:	Starter::start_polling_thread
- *
- *	description:	method to execute "StartPollingThread"
- *	Start the polling thread for DevState command.
- *
- *
- */
-//+------------------------------------------------------------------
-void Starter::start_polling_thread()
-{
-	INFO_STREAM << "Starter::start_polling_thread(): entering... !" << endl;
-
-	//	Notify thread that command has been received
-	//	Done in always_executed_hook
-	//if (shared !=NULL)
-	//	shared->set_timer();
+	INFO_STREAM << "Starter::processCouldStart(\""<< argin << "\"): entering... !" << endl;
 	
-	//	Done in always_executed_hook method
-}
-
-//+----------------------------------------------------------
-//
-//	WIN 32 methods to fork a sub process
-//
-//+----------------------------------------------------------
-#ifdef WIN32
-void StartWinProcess::initialize(char *server, char *instancename, char *log_file)
-{
-	this->server       = (char *)malloc(strlen(server)+1);
-	this->instancename = (char *)malloc(strlen(instancename)+1);
-	this->log_file     = (char *)malloc(strlen(log_file)+1);
-	strcpy(this->server, server);
-	strcpy(this->instancename, instancename);
-	strcpy(this->log_file, log_file);
-}
-//+----------------------------------------------------------
-//+----------------------------------------------------------
-void *StartWinProcess::run_undetached(void *ptr)
-{
-	//	Check if batch file
-	string	str_server(server);
-	if (str_server.find(".bat") != string::npos)
-	{
-		char	*argv[3];
-		int		i = 0;
-		argv[i++] = server;
-		argv[i++] = instancename;
-		argv[i++] = NULL;
-
-		cout << "Forking " << server << endl;
-		cout << "log file: " << log_file << endl;
-
-		//	Write a starting message in log file
-		ofstream	ofs1(log_file);
-		ofs1 << "Starting " << argv[0] << "....." << endl << endl;
-		ofs1.close();
-
-		//	Close the stderr and re-open it on a log file.
-		//-------------------------------------------------
-		int	ret;
-		if ((ret=_spawnv(_P_WAIT, argv[0], argv))<0)
-		{
-			//	Write error in log file
-			//-------------------------------
-			ofstream	ofs(log_file);
-			ofs << "Exec(" << argv[0] << ") failed " << endl;
-			ofs << strerror(errno) << endl;
-			ofs << "_spawnv has returned " << ret << endl;
-			ofs.close();
-		}
-		cout << "Fork Succeed." << endl;
-	}
-	else
-	{
-		PROCESS_INFORMATION pi;
-		STARTUPINFO si;
-
-		string	cmd_line(server);
-		cmd_line += " ";
-		cmd_line += instancename;
-
-		//	Prepare Startup info
-		memset( &si, 0, sizeof(si) );
-		si.cb = sizeof(si);
-		si.dwFlags     = STARTF_USESHOWWINDOW;
-		si.wShowWindow = SW_HIDE;
-
-		if( !CreateProcess(
-				NULL ,            // pointer to name of executable module
-				(char*)cmd_line.c_str(), // pointer to command line string
-				NULL,             // process security attributes
-				NULL,             // thread security attributes
-				FALSE,            // handle inheritance flag
-				DETACHED_PROCESS, // creation flags
-				NULL,             // pointer to new environment block
-				NULL,             // pointer to current directory name
-				&si,              // pointer to STARTUPINFO
-				&pi               // pointer to PROCESS_INFORMATION
-				) )
-		{
-			_sleep(2000);
-			cerr << "Error when trying to create process " << cmd_line <<endl << strerror(errno) << endl;
-		}
-		cout << cmd_line << endl;
-	}
-	return NULL;
-}
-#endif
-
-//+------------------------------------------------------------------
-/**
- *	method:	Starter::dev_start
- *
- *	description:	method to execute "DevStart"
- *	Start the specified server.
- *
- * @param	argin	Server to be started.
- *
- */
-//+------------------------------------------------------------------
-void Starter::dev_start(Tango::DevString argin)
-{
-	INFO_STREAM << "Starter::dev_start(\""<< argin << "\"): entering... !" << endl;
-
 	//	Make sure that it's not running.
 	//---------------------------------------
 	if (servers.size()>0)
 	{
-		ServerControled	*server = util->get_server_by_name(argin, servers);
+		string	name(argin);
+		ControledServer	*server = util->get_server_by_name(name, servers);
 		if (server!=NULL)
 			if (server->running)
 			{
@@ -964,14 +807,16 @@ void Starter::dev_start(Tango::DevString argin)
 								(const char *)"ALREADY_RUNNING",
 								out_stream.str(),
 								(const char *)"Starter::dev_start()");
-				return;
-		}
+				return NULL;
+			}
 	}
 
 	//	Separate server name and instancename.
 	//-------------------------------------
-	char	*servname     = util->get_server_name(argin);
+	char	*servname     = util->get_server_name(argin) ;
 	char	*instancename = util->get_instance_name(argin);
+	char	*adminname = new char[strlen(servname)+ strlen(instancename)+10];
+	sprintf(adminname, "dserver/%s/%s", servname, instancename);
 	char	*filename;
 	try {
 		filename = util->check_exe_file(servname, startDsPath);
@@ -982,7 +827,7 @@ void Starter::dev_start(Tango::DevString argin)
 		else
 		{
 			cout << e.errors[0].desc << endl;
-			return;
+			return NULL;
 		}
 	}
 
@@ -1022,86 +867,53 @@ void Starter::dev_start(Tango::DevString argin)
 				INFO_STREAM << logpath << " Created !" << endl;
 		}
 	}
-
-#ifdef WIN32
-	//	Start process for WIN32
-	//-------------------------------------
-	StartWinProcess	*pt = new StartWinProcess;
-	pt->initialize(filename, instancename, (char *)log_file.c_str());
-	pt->start();
-
-#else
-	//---------------------------------------
-	//	UNIX like system (fork subprocess)
-	//---------------------------------------
-
-	//	Prepeare server args before fork
-	//---------------------------------------
-	char	*argv[4];
-	int		i = 0;
-	argv[i++] = filename;
-	argv[i++] = instancename;
-	//argv[i++] = "-v5";
-	argv[i++] = NULL;
-
-	cout << "forking " << argv[0] << " " << argv[1] << endl;
-	cout << hex << (argv[0]) << endl;
-	//	Fork a child process to start the device server
-	switch(fork())
-	{
-	case -1:
-		cerr << "Fork Failed !" << endl;
-		break;
-	case 0:
-		switch(fork())
-		{
-		case -1:
-			cerr << "Fork Failed !" << endl;
-			break;
-		case 0:
-			{
-				// Change process group and close control tty
-				setpgrp();
-
-				//	Call setsid() to do NOT stop children if Starter is killed.
-				//--------------------------------------------------------------
-				setsid();
-
-				//	Close the stderr and re-open it on a log file.
-				//-------------------------------------------------
-				close(2);
-				remove((char*)log_file.c_str());
-				open((char*)log_file.c_str(), O_RDWR | O_CREAT, 0664);
-
-				//	Start the execution of the device server
-				//---------------------------------------------
-				if (execvp(argv[0], argv)<0)
-				{
-					ofstream	of((char *)log_file.c_str());
-					of << "Exec(" << argv[0] << ") failed " << endl;
-					of << strerror(errno) << endl;
-					of.close();
-				}
-				_exit(0);
-			}
-			break;
-			default:
-				cout << "Fork 2 Succeed !" << endl;
-				WARN_STREAM << argin << " started !" << endl;
-				_exit(0);
-				break;
-		}
-		break;
-
-	default:
-		cout << "Fork 1 Succeed !" << endl;
-		WARN_STREAM << argin << " started !" << endl;
-		int		res_wait;
-		wait(&res_wait);
-		break;
-	}
-#endif	//	WIN32 
+	
+	NewProcess	*np  = new NewProcess;
+	np->servname     = filename;
+	np->instancename = instancename;
+	np->adminname    = adminname;
+	np->logfile      = new char[log_file.length()+1];
+	np->logfile      = strcpy(np->logfile, log_file.c_str());
+	
+	return np;
 }
+//+------------------------------------------------------------------
+/**
+ *	method:	Starter::dev_start
+ *
+ *	description:	method to execute "DevStart"
+ *	Start the specified server.
+ *
+ * @param	argin	Server to be started.
+ *
+ */
+//+------------------------------------------------------------------
+void Starter::dev_start(Tango::DevString argin)
+{
+	INFO_STREAM << "Starter::dev_start(\""<< argin << "\"): entering... !" << endl;
+
+	NewProcess	*np = processCouldStart(argin);
+	if (np==NULL)
+		return;
+
+	//	Build a vector to start process
+	vector<NewProcess *>	processes;
+	processes.push_back(np);
+	startProcesses(processes, 0);
+}
+//+------------------------------------------------------------------
+//+------------------------------------------------------------------
+void Starter::startProcesses(vector<NewProcess *> v_np, int level)
+{
+	//	Start process to start processes
+	//-------------------------------------
+	start_process_thread_levels.push_back(level);
+	starting_processes++;
+	StartProcessThread	*pt =
+		new StartProcessThread(v_np, level, this);
+	pt->start();
+}
+
 //+------------------------------------------------------------------
 /**
  *	method:	Starter::dev_stop
@@ -1131,7 +943,8 @@ void Starter::dev_stop(Tango::DevString argin)
 
 	//	Check Argin as server name
 	//----------------------------------
-	ServerControled	*server = util->get_server_by_name(argin, servers);
+	string	name(argin);
+	ControledServer	*server = util->get_server_by_name(name, servers);
 	if (server==NULL)
 	{
 		TangoSys_MemStream out_stream;
@@ -1150,44 +963,16 @@ void Starter::dev_stop(Tango::DevString argin)
 		out_stream << argin << " is NOT running !" << ends;
 		Tango::Except::throw_exception(out_stream.str(),
 				out_stream.str(),
-				(const char *)"Starter::dev_start()");
+				(const char *)"Starter::dev_stop()");
 		return;
 	}
 	
 	//	And Kill it
 	//-------------------
-//	Tango::DeviceProxy	*dev = new Tango::DeviceProxy(server->admin_name);
-	if (server->dev == NULL)
-		server->dev = new Tango::DeviceProxy(server->admin_name);
-	try
-	{
-		//	increase timeout (minimum for ping)
-		server->dev->set_timeout_millis(2000);
-		server->dev->command_inout("Kill");
-		WARN_STREAM << argin << " stopped !" << endl;
-		cout << argin << " stopped !" << endl;
-		//	restore timeout for ping
-		server->dev->set_timeout_millis(PING_TIMEOUT);
-	}
-	catch(Tango::DevFailed &e)
-	{
-		for (int i=0 ; i<e.errors.length() ; i++)
-		{
-			string	reason(e.errors[0].reason);
-			if (reason=="API_CommandNotFound")
-			{
-				//	In old IDL version Kill was DevKill
-				server->dev->command_inout("DevKill");
-				//	restore timeout for ping
-				server->dev->set_timeout_millis(PING_TIMEOUT);
-				return;
-			}
-		}
-		//	restore timeout for ping
-		server->dev->set_timeout_millis(PING_TIMEOUT);
-		Tango::Except::print_exception(e);
-		throw e;
-	}
+	Tango::DeviceProxy	*dev = new Tango::DeviceProxy(server->admin_name);
+	dev->command_inout("Kill");
+	WARN_STREAM << argin << " stopped !" << endl;
+	cout << argin << " stopped !" << endl;
 }
 
 //+------------------------------------------------------------------
@@ -1293,7 +1078,7 @@ int	Starter::nb_servers_to_start(int level)
 	int	cnt = 0;
 	for (int i=0 ; i<servers.size() ; i++)
 	{
-		ServerControled	*server = &servers[i];
+		ControledServer	*server = &servers[i];
 		//	server->running could not be initialized
 		if (server->controled  &&  server->startup_level==level)
 			if (is_running(server)==false)
@@ -1339,32 +1124,24 @@ void Starter::dev_start_all(Tango::DevShort level)
 
 	//	And start the stopped ones
 	//---------------------------------------------------
+	vector<NewProcess *>	processes;
 	for (int i=0 ; i<servers.size() ; i++)
 	{
-		ServerControled	*server = &servers[i];
+		ControledServer	*server = &servers[i];
 		//	server->running could not be initialized
-		INFO_STREAM << "checking " << server->name << endl;
 		if (server->controled  &&  server->startup_level==level)
 		{
 			server->running = is_running(server);
 			if (server->running==false)
 			{
-				dev_start((char*)server->name.c_str());
-
-				//	Sleep a bit between startups
-				//	to do not overload database
-#			ifdef WIN32
-				_sleep(TIME_BETWEEN_STARTUPS);
-#			else
-				struct timespec ts;
-				ts.tv_sec = 0;
-				ts.tv_nsec = TIME_BETWEEN_STARTUPS * 1000;
-				nanosleep(&ts,NULL);
-#			endif
+				NewProcess	*np = processCouldStart((char*)server->name.c_str());
+				if (np!=NULL)
+					processes.push_back(np);
 			}
 		}
-		INFO_STREAM << "check " << server->name << " done." << endl;
 	}
+	if (processes.size()>0)
+		startProcesses(processes, level);
 
 	//	Want exception during normal run
 	throwable = true;
@@ -1400,7 +1177,7 @@ void Starter::dev_stop_all(Tango::DevShort level)
 	//---------------------------------------------------
 	for (int i=0 ; i<servers.size() ; i++)
 	{
-		ServerControled	*server = &servers[i];
+		ControledServer	*server = &servers[i];
 		if (server->controled             &&
 			server->startup_level==level  &&
 			server->running==true)
@@ -1468,8 +1245,6 @@ Tango::ConstDevString Starter::dev_read_log(Tango::DevString argin)
 	ifs.close();
 	returned_str = strlog.str();
 	return returned_str.c_str();
-
-//	return "";
 }
 
 //+------------------------------------------------------------------
