@@ -1,6 +1,6 @@
 static const char *TagName   = "$Name:  $";
 static const char *HttpServer= "http://www.esrf.fr/computing/cs/tango/tango_doc/ds_doc/";
-static const char *RcsId = "$Header: /cvsroot/tango-cs/tango/cppserver/starter/StarterClass.cpp,v 3.9 2005/09/30 12:01:50 pascal_verdier Exp $";
+static const char *RcsId = "$Header: /cvsroot/tango-cs/tango/cppserver/starter/StarterClass.cpp,v 3.16 2006/06/13 19:38:15 pascal_verdier Exp $";
 //+=============================================================================
 //
 // file :        StarterClass.cpp
@@ -14,9 +14,32 @@ static const char *RcsId = "$Header: /cvsroot/tango-cs/tango/cppserver/starter/S
 //
 // $Author: pascal_verdier $
 //
-// $Revision: 3.9 $
+// $Revision: 3.16 $
 //
 // $Log: StarterClass.cpp,v $
+// Revision 3.16  2006/06/13 19:38:15  pascal_verdier
+// Minor changes.
+//
+// Revision 3.15  2006/06/06 12:01:26  pascal_verdier
+// Bug in log file fixed.
+//
+// Revision 3.14  2006/06/05 07:20:36  pascal_verdier
+// Server startup is now delayed (with timeout) in a startup level.
+// New state MOVING added.
+// At startup, starter loop until properties have been read.
+//
+// Revision 3.13  2006/05/15 11:11:07  pascal_verdier
+// Tag Release added to const char *TagName.
+//
+// Revision 3.12  2006/04/24 08:58:10  pascal_verdier
+// *** empty log message ***
+//
+// Revision 3.11  2006/04/24 07:06:28  pascal_verdier
+// A thread is started for each level when at servers startup.
+//
+// Revision 3.10  2006/02/08 07:13:55  pascal_verdier
+// Minor changes.
+//
 // Revision 3.9  2005/09/30 12:01:50  pascal_verdier
 // RunningServers and StoppedServers spectrum attribute max_x set to 200.
 //
@@ -128,27 +151,6 @@ CORBA::Any *NotifyDaemonStateCmd::execute(Tango::DeviceImpl *device,const CORBA:
 }
 
 
-//+----------------------------------------------------------------------------
-//
-// method : 		StartPollingThreadCmd::execute()
-// 
-// description : 	method to trigger the execution of the command.
-//                PLEASE DO NOT MODIFY this method core without pogo   
-//
-// in : - device : The device on which the command must be excuted
-//		- in_any : The command input data
-//
-// returns : The command output data (packed in the Any object)
-//
-//-----------------------------------------------------------------------------
-CORBA::Any *StartPollingThreadCmd::execute(Tango::DeviceImpl *device,const CORBA::Any &in_any)
-{
-
-	cout2 << "StartPollingThreadCmd::execute(): arrived" << endl;
-
-	((static_cast<Starter *>(device))->start_polling_thread());
-	return new CORBA::Any();
-}
 
 //+----------------------------------------------------------------------------
 //
@@ -471,11 +473,6 @@ void StarterClass::command_factory()
 		"",
 		"",
 		Tango::OPERATOR));
-	command_list.push_back(new StartPollingThreadCmd("StartPollingThread",
-		Tango::DEV_VOID, Tango::DEV_VOID,
-		"",
-		"",
-		Tango::OPERATOR));
 	command_list.push_back(new NotifyDaemonStateCmd("NotifyDaemonState",
 		Tango::DEV_VOID, Tango::DEV_STATE,
 		"",
@@ -617,6 +614,7 @@ void StarterClass::get_class_property()
 	cl_prop.push_back(Tango::DbDatum("NbStartupLevels"));
 	cl_prop.push_back(Tango::DbDatum("CmdPollingTimeout"));
 	cl_prop.push_back(Tango::DbDatum("UseEvents"));
+	cl_prop.push_back(Tango::DbDatum("ServerStartupTimeout"));
 
 	//	Call database and extract values
 	//--------------------------------------------
@@ -652,6 +650,13 @@ void StarterClass::get_class_property()
 	else
 		cl_prop[i]  <<  useEvents;
 
+	//	Extract ServerStartupTimeout value
+	def_prop = get_default_class_property(cl_prop[++i].name);
+	if (def_prop.is_empty()==false)	def_prop  >>  serverStartupTimeout;
+	if (cl_prop[i].is_empty()==false)	cl_prop[i]  >>  serverStartupTimeout;
+	else
+		cl_prop[i]  <<  serverStartupTimeout;
+
 	//	End of Automatic code generation
 	//-------------------------------------------------------------
 	cout2 << "readInfoDbPeriod  = " << readInfoDbPeriod << endl;
@@ -734,6 +739,21 @@ void StarterClass::set_default_property()
 	else
 		add_wiz_class_prop(prop_name, prop_desc);
 
+	prop_name = "ServerStartupTimeout";
+	prop_desc = "Timeout on device server startup in seconds.";
+	prop_def  = "1";
+	vect_data.clear();
+	vect_data.push_back("1");
+	if (prop_def.length()>0)
+	{
+		Tango::DbDatum	data(prop_name);
+		data << vect_data ;
+		cl_def_prop.push_back(data);
+		add_wiz_class_prop(prop_name, prop_desc,  prop_def);
+	}
+	else
+		add_wiz_class_prop(prop_name, prop_desc);
+
 	//	Set Default Device Properties
 	prop_name = "StartDsPath";
 	prop_desc = "Path to find executable files\nto start device servers";
@@ -808,6 +828,21 @@ void StarterClass::set_default_property()
 	else
 		add_wiz_dev_prop(prop_name, prop_desc);
 
+	prop_name = "ServerStartupTimeout";
+	prop_desc = "Timeout on device server startup in seconds.";
+	prop_def  = "5";
+	vect_data.clear();
+	vect_data.push_back("5");
+	if (prop_def.length()>0)
+	{
+		Tango::DbDatum	data(prop_name);
+		data << vect_data ;
+		dev_def_prop.push_back(data);
+		add_wiz_dev_prop(prop_name, prop_desc,  prop_def);
+	}
+	else
+		add_wiz_dev_prop(prop_name, prop_desc);
+
 }
 //+----------------------------------------------------------------------------
 //
@@ -873,7 +908,7 @@ void StarterClass::write_class_property()
 	start = header.length();
 	string	endstr(" $");
 	end   = tagname.find(endstr);
-	if (end>start)
+	if (end!=string::npos && end>start)
 	{
 		string	strtag = tagname.substr(start, end-start);
 		Tango::DbDatum	cvs_tag("cvs_tag");
