@@ -1,4 +1,4 @@
-static const char *RcsId = "$Header: /cvsroot/tango-cs/tango/cppserver/starter/StarterUtil.cpp,v 3.7 2005/09/19 13:35:22 pascal_verdier Exp $";
+static const char *RcsId = "$Header: /cvsroot/tango-cs/tango/cppserver/starter/StarterUtil.cpp,v 3.10 2006/06/05 07:20:36 pascal_verdier Exp $";
 //+=============================================================================
 //
 // file :         StarterUtil.cpp
@@ -9,9 +9,20 @@ static const char *RcsId = "$Header: /cvsroot/tango-cs/tango/cppserver/starter/S
 //
 // $Author: pascal_verdier $
 //
-// $Revision: 3.7 $
+// $Revision: 3.10 $
 //
 // $Log: StarterUtil.cpp,v $
+// Revision 3.10  2006/06/05 07:20:36  pascal_verdier
+// Server startup is now delayed (with timeout) in a startup level.
+// New state MOVING added.
+// At startup, starter loop until properties have been read.
+//
+// Revision 3.9  2006/04/24 07:06:28  pascal_verdier
+// A thread is started for each level when at servers startup.
+//
+// Revision 3.8  2006/02/09 11:59:19  pascal_verdier
+// A ping thread is now started for each server.
+//
 // Revision 3.7  2005/09/19 13:35:22  pascal_verdier
 // change on traces.
 //
@@ -142,12 +153,15 @@ string StarterUtil::removeFQDN(string s)
 //+------------------------------------------------------------------
 char *StarterUtil::get_server_name(char *argin)
 {
-	static char servname[50];
+	char	*servname;
+	char	tmp[50];
 	char	*p1 = argin;
-	char	*p2 = servname;	
+	char	*p2 = tmp;	
 	while (*p1 && *p1!='/')
 		*p2++ = *p1++;
 	*p2++ = '\0';
+	servname = (char *)malloc(strlen(tmp)+1);
+	strcpy(servname, tmp);
 	return servname;
 }
 //+------------------------------------------------------------------
@@ -157,9 +171,10 @@ char *StarterUtil::get_server_name(char *argin)
 //+------------------------------------------------------------------
 char *StarterUtil::get_instance_name(char *argin)
 {
-	static char instancename[50];
+	char	*instancename;
+	char	tmp[50];
 	char	*p1 = argin;
-	char	*p2 = instancename;	
+	char	*p2 = tmp;	
 	while (*p1 && *p1!='/')
 		p1++;
 	p1++;
@@ -167,6 +182,8 @@ char *StarterUtil::get_instance_name(char *argin)
 		*p2++ = *p1++;
 	*p2++ = '\0';
 	
+	instancename = (char *)malloc(strlen(tmp)+1);
+	strcpy(instancename, tmp);
 	return instancename;
 }
 //+----------------------------------------------------------------------------
@@ -310,7 +327,7 @@ string StarterUtil::build_log_file_name(char *server)
 	//-----------------------------------------
 	string	log_file;
 	LogPath(log_file);
-	log_file += "/";
+	log_file += slash;
 	log_file += servname;
 	log_file += "_";
 	log_file += intancename;
@@ -414,7 +431,7 @@ vector<string>	StarterUtil::get_host_ds_list()
  *	@param	server	object to be updated from db read.
  */
 //+------------------------------------------------------------------
-void StarterUtil::get_server_info(ServerControled *server)
+void StarterUtil::get_server_info(ControledServer *server)
 {
 	try
 	{
@@ -441,189 +458,107 @@ void StarterUtil::get_server_info(ServerControled *server)
  *	Allocate and fill the servers controled object
  */
 //+------------------------------------------------------------------
-void StarterUtil::build_server_ctrl_object(vector<ServerControled> *servers)
+void StarterUtil::build_server_ctrl_object(vector<ControledServer> *servers)
 {
-	cout << "build_server_ctrl_object()" << endl;
-	//	The first time, check if DbGetHostServersInfo command
-	//	is implemented in this database server version.
-	//-------------------------------------------------------------
-	static bool	one_call = true;
+	bool trace = false;
+	if (trace)	cout << "build_server_ctrl_object()" << endl;
+	int	i;
 	vector<string>	result;
-	if (one_call)
+	for (i=0 ; i<hostnames.size() ; i++)
 	{
-		try
-		{
-			for (int i=0 ; i<hostnames.size() ; i++)
-			{
-				//	Call for servers and their info for each host
-				cout << "Call for servers on " << hostnames[i] << endl;
-				Tango::DeviceData	argin;
-				argin << hostnames[i];
-				Tango::DeviceData	argout = dbase->command_inout("DbGetHostServersInfo", argin);
-				vector<string>	tmp;
-				argout >> tmp;
-				//	Copy to global vector
-				for (int j=0 ; j<tmp.size() ; j++)
-					result.push_back(tmp[j]);
-			}
-
-		}
-		catch(Tango::DevFailed &e)
-		{
-			Tango::Except::print_exception(e);
-			//	Check if it is due to a prvious DatabaseDs version
-			string  reason(e.errors[0].reason);
-			if (reason=="API_CommandNotFound")
-				one_call = false;
-			else
-				throw e;
-		}
+		//	Call for servers and their info for each host
+		cout << "Call for servers on " << hostnames[i] << endl;
+		Tango::DeviceData	argin;
+		argin << hostnames[i];
+		Tango::DeviceData	argout = dbase->command_inout("DbGetHostServersInfo", argin);
+		argout >> result;
 	}
-	if (one_call)
+
+	if (trace)	cout << "--------------  Check if list of servers modified  --------------" << endl;
+	//	Check servers really used (erase this one and database server
+	Tango::Util *tg = Tango::Util::instance();
+	vector<string>::iterator pos;
+	for (pos=result.begin() ; pos<result.end() ; pos+=3)
 	{
-		//	Check servers really used (erase this one and database server
-		Tango::Util *tg = Tango::Util::instance();
-		vector<string>::iterator pos;
-		for (pos=result.begin() ; pos<result.end() ; pos+=3)
+		int	idx = (*pos).find_first_of("/");
+		if (idx>0)
 		{
-			int	idx = (*pos).find_first_of("/");
-			if (idx>0)
+			//	Get process name only in lower case before compeare
+			string	s = (*pos).substr(0, idx);
+			transform(s.begin(), s.end(), s.begin(), ::tolower);
+			if (s=="starter"  ||  s=="databaseds" || s=="logconsumer")
 			{
-				//	Get process name only in lower case before compeare
-				string	s = (*pos).substr(0, idx);
-				transform(s.begin(), s.end(), s.begin(), ::tolower);
-				if (s=="starter"  ||  s=="databaseds" || s=="logconsumer")
-				{
-					result.erase(pos);
-					result.erase(pos);
-					result.erase(pos);
-					pos -=3;	//	because erase decrease size !
-				}
+				result.erase(pos);
+				result.erase(pos);
+				result.erase(pos);
+				pos -=3;	//	because erase decrease size !
 			}
 		}
-		//	If nb servers not modified
-		//--------------------------------------------------------------------
-		int		nb_serv = result.size() / 3; //	name & 2 info
-		int		i;
-		bool	modif = false;
-		if (nb_serv!=servers->size())
-			modif = true;
-		else
-			//	Check if list of servers modified
-			//--------------------------------------------------------
-			for (i=0 ; i<servers->size() ; i++)
-				if (i<result.size())
-					if ((*servers)[i].name!=result[3*i])
-						modif = true;
-		//	If modified re-create object.
-		//--------------------------------------------------------
-		if (modif)
-		{
-			cout << "----------------------------------------" << endl;
-			cout << (result.size()/3) << " servers found" << endl;
-			servers->clear();
-			//	Create Server  controled object
-			//---------------------------------------------------------
-			for (i=0 ; i<result.size() ; i+=3)
-			{
-				ServerControled	server;
+	}
+	//	Check if some servers have disappeared
+	vector<ControledServer>::iterator it;
+	for (it=servers->begin() ; it<servers->end() ; it++)
+	{
+		string	s1(it->name);
+		bool	found = false;
+		for (i=0 ; !found && i<result.size() ; i+=3)
+			found = (s1==result[i]);
 
-				//	get names
-				//-------------------------
-				server.name = result[i];
-				server.admin_name = "dserver/" + server.name;
-				server.dev = NULL;
-
-				servers->push_back(server);
-				cout << "\t" << (i/3) << " - " <<  server.name	<< endl;
-			}
-		}
-		//	Set if controlled or not
-		for (i=0 ; i<servers->size() ; i++)
+		if (!found)
 		{
-			(*servers)[i].controled     = (atoi(result[3*i+1].c_str())==0)? false: true;
-			(*servers)[i].startup_level =  atoi(result[3*i+2].c_str());
+			if (trace)	cout << s1 << " has disappeared" << endl;
+			//	if disappeared then stop thread and remove reference
+			it->thread_data->set_stop_thread();
+			servers->erase(it);
 		}
 	}
-	if (one_call==false)
+	//	Check if new servers appeared
+	for (pos=result.begin() ; pos<result.end() ; )
 	{
-		build_server_ctrl_object_multi_call(servers);
-		return;
-	}
-}
-//+------------------------------------------------------------------
-/**
- *	Allocate and fill the servers controled object
- */
-//+------------------------------------------------------------------
-void StarterUtil::build_server_ctrl_object_multi_call(vector<ServerControled> *servers)
-{
-	//	Get the list of the servers running on this host
-	//----------------------------------------------------
-	vector<string>	servnames;
-	servnames  = get_host_ds_list();
-
-	//	Get the number of servers (if none, nothing to do)
-	//--------------------------------------------------------
-	if (servnames.size()==0)
-		return;
-
-	//	If nb servers not modified
-	//--------------------------------------------------------------------
-	int		i;
-	bool	modif = false;
-	if (servnames.size()!=servers->size())
-		modif = true;
-	else
-		//	Check if list of servers modified
-		//--------------------------------------------------------
-		for (i=0 ; i<servnames.size() ; i++)
-			if (i<servers->size())
-				if ((*servers)[i].name!=servnames[i])
-					modif = true;
-
-	//	If modified re-create object.
-	//--------------------------------------------------------
-	if (modif)
-	{
-		servers->clear();
-		//	Create Server  controled object
-		//---------------------------------------------------------
-		cout2 << "------------->   Allocate memory for object !!!" << endl;
-		for (i=0 ; i<servnames.size() ; i++)
+		string	name(*pos++);
+		ControledServer	*p_serv = get_server_by_name(name, *servers);
+		if (p_serv==NULL)
 		{
-			ServerControled	server;
+			if (trace)	cout << name << " appeared " << endl;
+			//	Create a new server instance
+			ControledServer	server;
 
-			//	get names
-			//-------------------------
-			server.name = servnames[i];
+			server.name = name;
 			server.admin_name = "dserver/" + server.name;
 			server.dev = NULL;
+			server.controled = (atoi((*pos++).c_str())==0)? false: true;
+			server.startup_level =  atoi((*pos++).c_str());
+			server.running = true;
+
+			//	Add a thread to ping server
+			server.thread_data = new PingThreadData(server.name);
+			server.thread = new PingThread(server.thread_data, server.name);
+			server.thread->start();
 
 			servers->push_back(server);
+			ms_sleep(50);
+		}
+		else
+		{
+			//	Update levels
+			p_serv->controled     = (atoi((*pos++).c_str())==0)? false: true;
+			p_serv->startup_level =  atoi((*pos++).c_str());
 		}
 	}
-
-	//	Set if controlled or not
-	for (i=0 ; i<servnames.size() ; i++)
-		get_server_info(&((*servers)[i]));
 }
 
-
-
-
 //+------------------------------------------------------------------
 /**
- *	search a server in ServerControled array by it's name .
+ *	search a server in ControledServer array by it's name .
  *
  *	@param servname	Server searched name.
  */
 //+------------------------------------------------------------------
-ServerControled *StarterUtil::get_server_by_name(string servname, vector<ServerControled> &servers)
+ControledServer *StarterUtil::get_server_by_name(string &servname, vector<ControledServer> &servers)
 {
 	for (int i=0 ; i<servers.size() ; i++)
 	{
-		ServerControled	*server = &servers[i];
+		ControledServer	*server = &servers[i];
 		if (server->name == servname)
 			return server;
 	}
@@ -723,8 +658,8 @@ Tango::DevState StarterUtil::is_notifyd_alive()
 	}
 	catch (...)
 	{
-		cout << "Failed to narrow the EventChannelFactory on " << notifyd_name << endl;
-		cout << " make sure the notifd process is running on this host" << endl;
+		cerr << "Failed to narrow the EventChannelFactory on " << notifyd_name << endl;
+		cerr << " make sure the notifd process is running on this host" << endl;
 		ch_factory = NULL;
  		return Tango::DevState(Tango::FAULT);
 	}
@@ -745,139 +680,71 @@ Tango::DevState StarterUtil::is_notifyd_alive()
 
 //+----------------------------------------------------------------------------
 /**
- *	Consdstructor for Shared data object 
+ *	Create a thread to start  polling.
  */
 //+----------------------------------------------------------------------------
-SharedData::SharedData()
+PollingState::PollingState(string devname)
 {
-	//	Get the polling thread ID
-	//Tango::Util *tg = Tango::Util::instance();
-	//polling_id = tg->get_polling_thread_object();
-	
+	device_name = devname;
 }
 //+----------------------------------------------------------------------------
 /**
- *	Return the elapsed time from last command.
- */
-//+----------------------------------------------------------------------------
-time_t SharedData::get_timer()
-{
-	omni_thread	*this_thread = omni_thread::self();
-	cout2 << "Thread : " << this_thread << " - SharedData::get_timer()" << endl;
-	omni_mutex_lock sync(*this);
-	//	A Command has been received -> reset timer
-	time_t	retval = time(NULL) - last_cmd_time;
-	cout2 << "Thread : " << this_thread << " SharedData::get_timer() done" << endl;
-	return retval;
-}
-//+----------------------------------------------------------------------------
-/**
- *	Set the timer to curent time (a command has been received).
- */
-//+----------------------------------------------------------------------------
-void SharedData::set_timer()
-{
-	omni_thread	*this_thread = omni_thread::self();
-	cout2 << "Thread : " << this_thread << " - SharedData::set_timer()" << endl;
-	omni_mutex_lock sync(*this);
-	//	Do it only if command is not form polling thread
-	//omni_thread	*this_thread = omni_thread::self();
-	//cout2 << "SharedData::set_timer() - thread: " << this_thread << endl;
-	//if (this_thread != polling_id)
-	//{
-		//	A Command has been received -> reset timer
-		last_cmd_time = time(NULL);
-		signal();
-	//}
-	//else
-	//	cout2 << "SharedData::set_timer() not done" << endl;
-	cout2 << "Thread : " << this_thread << " - SharedData::set_timer() done" << endl;
-}
-//+----------------------------------------------------------------------------
-/**
- *	Create a thread to start or stop state polling.
- *	Start will be done from astor.
- *	Stop will be done from astor or from a timeout without command.
- */
-//+----------------------------------------------------------------------------
-PollingState::PollingState(SharedData *sd, long timeout)
-{
-	shared      = sd;
-	cmd_timeout = timeout;
-}
-//+----------------------------------------------------------------------------
-/**
- *	Execute the thread loop.
- *	This thread is awaken when a command has been received 
- *	and falled asleep when no command has been received from a long time.
+ *	Execute the thread commands
  */
 //+----------------------------------------------------------------------------
 void *PollingState::run_undetached(void *ptr)
 {
-	Tango::Util 	*tg = Tango::Util::instance();
-	Tango::DServer	*dev = tg->get_dserver_device();
-	CORBA::Any		any;
-	omni_thread	*this_thread = omni_thread::self();
-
-	polling_on = false;
-	while(true)
-	{
-		time_t	delay = shared->get_timer();
-		cout2 << " PollingState::run() alive.  -  " << delay  << endl;
-
-		//	If a command has been received and polling not started
-		//			then START it
-		if (delay<cmd_timeout  && polling_on==false)
-		{
-			cout2 << "	Start polling " << endl;
-			try
-			{
-				dev->command_inout("StartPolling", any);
-				polling_on = true;
-			}
-			catch(Tango::DevFailed &e)
-			{
-				Tango::Except::print_exception(e);
-			}
-		}
-#ifdef NO_EVENTS
-		else
-		//	If a command has not been received for a long time 
-		//	and polling started   -  > then STOP it
-		if (delay>=cmd_timeout  && polling_on==true)
-		{
-			cout1 << "	Stop polling " << endl;
-			try
-			{
-				dev->command_inout("StopPolling", any);
-				polling_on = false;
-			}
-			catch(Tango::DevFailed &e)
-			{
-				Tango::Except::print_exception(e);
-			}
-		}
-		
-		//	And wait a bit if pollin on 
-		//	Or wait until next command.
-		{
-			omni_mutex_lock sync(*shared);
-			if (polling_on==true)
-				shared->wait(cmd_timeout * 1000);
-			else
-				shared->wait();
-		}
-		cout2 << "Thread : " << this_thread << " awaken" << endl;
-#else
-		//	Since events are used DO NOT STOP POLLING
-		cout << "PollingState::run_undetached() exiting" << endl;
-		return NULL;
-#endif
-		
-	}
+	//	wait a bit for dserver exported.
+	ms_sleep(1000);
+	long	upd_period = 1000;
+	add_obj_polling("attribute", "State",          upd_period);
+	add_obj_polling("attribute", "HostState",      upd_period);
+	add_obj_polling("attribute", "RunningServers", upd_period);
+	add_obj_polling("attribute", "StoppedServers", upd_period);
 	return NULL;
 }
  
+//+------------------------------------------------------------------
+/**
+ *	Add the Polling object.
+ */
+//+------------------------------------------------------------------
+void PollingState::add_obj_polling(string type, string objname, int upd_period)
+{
+	Tango::Util *tg = Tango::Util::instance();
+	Tango::DServer		*adm_dev = tg->get_dserver_device();
+	Tango::DevVarLongStringArray	*lsa = new Tango::DevVarLongStringArray();
+	lsa->lvalue.length(1);
+	lsa->svalue.length(3);
+	lsa->lvalue[0] = upd_period;
+	lsa->svalue[0] = CORBA::string_dup(device_name.c_str());
+	lsa->svalue[1] = CORBA::string_dup(type.c_str());
+	lsa->svalue[2] = CORBA::string_dup(objname.c_str());
+	CORBA::Any	any;
+	any <<= lsa;
+	try {
+		adm_dev->command_inout("AddObjPolling", any);
+	}
+	catch (Tango::DevFailed &e) {
+
+		string	reason(e.errors[0].reason);
+		if (reason=="API_AlreadyPolled")
+		{
+			//	If already polled -> just update period.
+			//----------------------------------------------
+			try {
+				adm_dev->command_inout("UpdObjPollingPeriod", any);
+				//cout <<"UpdObjPollingPeriod(" << objname <<
+				//			", " << upd_period << " ms)" << endl;
+			}
+			catch (Tango::DevFailed &e) {
+				Tango::Except::print_exception(e);
+			}
+		}
+		else
+			Tango::Except::print_exception(e);
+	}
+}
 //+----------------------------------------------------------------------------
 
 
