@@ -8,7 +8,15 @@
 //
 // $Author: pascal_verdier $
 //
-// $Revision: 3.6 $
+// $Revision: 3.8 $
+//
+// $Log: StarterUtil.h,v $
+// Revision 3.8  2006/04/24 07:06:28  pascal_verdier
+// A thread is started for each level when at servers startup.
+//
+// Revision 3.7  2006/02/09 11:59:19  pascal_verdier
+// A ping thread is now started for each server.
+//
 //
 // copyleft :    European Synchrotron Radiation Facility
 //               BP 220, Grenoble 38043
@@ -19,10 +27,11 @@
 #define _STARTER_UTIL_H
 
 #include <tango.h>
+#include <PingThread.h>
 
 /**
  * @author	$Author: pascal_verdier $
- * @version	$Revision: 3.6 $
+ * @version	$Revision: 3.8 $
  */
 
 /**
@@ -45,9 +54,24 @@ typedef struct {
 	bool	controled;
 	bool	running;
 	short	startup_level;
+	PingThreadData	*thread_data;
+	PingThread		*thread;
 	Tango::DeviceProxy	*dev;
 }
-ServerControled;
+ControledServer;
+
+//	Millisecond sleep platform independant.
+//--------------------------------------------
+#	ifdef WIN32
+#	define		ms_sleep(ms)	_sleep(ms);
+#	else
+#	define		ms_sleep(ms)	{\
+		struct timespec ts;     \
+		ts.tv_sec = ms / 1000;  \
+		ts.tv_nsec = (ms - (ts.tv_sec * 1000)) * 1000000; \
+		nanosleep(&ts,NULL); \
+	}
+#	endif
 
 //	Definitions for separators
 //----------------------------------
@@ -63,6 +87,8 @@ ServerControled;
 		s += slash;		\
 		s += "ds.log";
 
+class Starter;
+
 class StarterUtil
 {
 public :
@@ -128,12 +154,12 @@ char *get_file_date(char *filename);
  */
 char *strtime(time_t t);
 /**
- *	search a server in ServerControled array by it's name in an array.
+ *	search a server in ControledServer array by it's name in an array.
  *
  *	@param servname	Server searched name.
  *	@param servers	Array of structure to search name.
  */
-ServerControled *get_server_by_name(string, vector<ServerControled>&);
+ControledServer *get_server_by_name(string&, vector<ControledServer>&);
 /**
  *	Get host device servers list from database.
  *
@@ -148,17 +174,17 @@ vector<string>	get_host_ds_list();
  *	@param	devname	device to get info.
  *	@param	server	object to be updated from db read.
  */
-void get_server_info(ServerControled *);
+void get_server_info(ControledServer *);
 /**
  *	Allocate and fill the servers controled object
  */
-void build_server_ctrl_object(vector<ServerControled> *servers);
-void build_server_ctrl_object_multi_call(vector<ServerControled> *servers);
+void build_server_ctrl_object(vector<ControledServer> *servers);
 /**
  *	check if Notify Daemon is alive.
  */
 Tango::DevState is_notifyd_alive();
 void import_notifyd();
+
 #ifdef WIN32
 /**
  *	This method does not exist in visual cpp
@@ -232,28 +258,18 @@ void set_timer();
 class PollingState: public omni_thread
 {
 private:
+	string	device_name;
+
 	/**
-	 *	Shared data
-	 */
-	SharedData		*shared;
-	/**
-	 *	status of polling
-	 */
-	Tango::DevBoolean	polling_on;
-	/**
-	 *	Timeout without command to fall asleep loop.
-	 */
-	Tango::DevLong		cmd_timeout;
+	 *	Add the object to polling list.
+ 	 */
+	void add_obj_polling(string type, string name, int period);
+
 public:
 /**
- *	Create a thread to start or stop state polling.
- *	Start will be done from astor.
- *	Stop will be done from astor or from a timeout without command.
- *
- *	@param	shared pointer on shared data between thread and DS.
- *	@param	timeout	timeout value in seconds.
+ *	Create a thread to start  polling.
  */
-	PollingState(SharedData *shared, long timeout);
+	PollingState(string);
 
 /**
  *	Execute the thread loop.
