@@ -30,6 +30,7 @@ class Database;
 class AsynReq;
 class EventConsumer;
 class CallBack;
+class AttributeProxy;
 
 class ApiUtilExt;
 class DeviceDataExt;
@@ -57,10 +58,14 @@ public:
 class ConnectionExt
 {
 public:
-	ConnectionExt():tr_reco(false) {};
+	ConnectionExt():tr_reco(true),prev_failed(false),prev_failed_t0(0.0) {};
+	ConnectionExt & operator=(const ConnectionExt &);
 	
 	bool			tr_reco;
 	Tango::Device_3_var 	device_3;
+	
+	bool			  prev_failed;
+	double		  prev_failed_t0;
 };
 
 class ApiUtilExt
@@ -853,6 +858,8 @@ private :
 	
 	void read_attr_except(CORBA::Request_ptr,long,read_attr_type);
 	void write_attr_except(CORBA::Request_ptr,long,TgRequest::ReqType);
+	
+	friend class AttributeProxy;
 			
 protected :
 	virtual string get_corba_name();
@@ -877,6 +884,9 @@ public :
 	DeviceProxy(const DeviceProxy &);
 	DeviceProxy & operator=(const DeviceProxy &);
 	~DeviceProxy();
+	
+	DeviceProxy():Connection((CORBA::ORB *)NULL),adm_device(NULL),ext_proxy(NULL)
+	{dbase_used = false;}
 
 //
 // general methods
@@ -1051,11 +1061,13 @@ private :
 	AttributeProxyExt	*ext;     	// Class extension
 
 	void real_constructor(string &);
-
+	void ctor_from_dp(const DeviceProxy *,string &);
 	
 public :
 	AttributeProxy(string &name);
 	AttributeProxy(const char *);
+	AttributeProxy(const DeviceProxy *,string &);
+	AttributeProxy(const DeviceProxy *,const char *);
 	AttributeProxy(const AttributeProxy &);
 	AttributeProxy & operator=(const AttributeProxy &);
 	~AttributeProxy();
@@ -1070,6 +1082,8 @@ public :
 	string status();
 	DevState state();	
 	int ping();
+	void set_transparency_reconnection(bool);
+	bool get_transparency_reconnection();
 	
 //
 // property methods
