@@ -12,9 +12,32 @@
 //
 // author(s) :		A.Gotz + E.Taurel
 //
-// $Revision: 3.5 $
+// $Revision: 3.10 $
 //
 // $Log: attrdesc.h,v $
+// Revision 3.10  2006/06/20 08:00:04  jensmeyer
+// After changing the the init value of the attribute polling period to -1 insted of 0 I detected
+// a campatibility problem with the old versions. I changed back the init value to 0 and
+// solved the problem with new methods on the Attribute object.
+//
+// Revision 3.9  2006/06/14 11:21:35  jensmeyer
+// Modifications to make the set_polling_period() for attributes work
+// with a polling period set to 0.
+//
+// Revision 3.8  2006/04/27 13:44:51  taurel
+// - Fix compatibility bug introduced in previous release by moving new
+// class data members in their corresponding extension classes
+//
+// Revision 3.7  2006/04/13 14:49:37  jensmeyer
+// Added methods to the Attr class to initialise the fire event configuration
+// with pogo.
+//
+// Revision 3.6  2005/11/07 12:15:00  jensmeyer
+// Corrected initialisation of memorised attributes. No longer exits when receiving an
+// exception during initialisation. Also added an option to only initialise the attribute setpoint and not writing to the attribute.
+// set_memorized_init(false) -> only applies setpoint.
+// set_memorized_init(true) -> also writes setpoint value to the attribute.
+//
 // Revision 3.5  2005/01/13 08:27:32  taurel
 // - Merge trunk with Release_5_0 from brach Release_5_branch
 //
@@ -189,10 +212,15 @@ class AttrExt
 {
 public:
 	AttrExt():poll_period(0) {disp_level = Tango::OPERATOR;}
-	AttrExt(DispLevel level) {disp_level = level;}
+	AttrExt(DispLevel level):poll_period(0) {disp_level = level;}
 	
 	Tango::DispLevel	disp_level;		// Display  level
 	long			poll_period;		// Polling period
+
+	bool			fire_change_event;
+	bool			fire_archive_event;	
+	bool			check_change_event;
+	bool			check_archive_event;
 };
 
 class UserDefaultAttrPropExt
@@ -208,8 +236,8 @@ class UserDefaultAttrPropExt
  * this UserDefaultAttrProp class) and finally a Tango library default
  * value
  *
- * $Author: taurel $
- * $Revision: 3.5 $
+ * $Author: jensmeyer $
+ * $Revision: 3.10 $
  */
  
 class UserDefaultAttrProp
@@ -461,8 +489,8 @@ public:
  * database allows the Tango core software to create the Attribute object
  * for the attribute created by the user.
  *
- * $Author: taurel $
- * $Revision: 3.5 $
+ * $Author: jensmeyer $
+ * $Revision: 3.10 $
  */
  
 class Attr
@@ -535,9 +563,70 @@ public:
 /**
  * Set the attribute as memorized in database (only for scalar and writable
  * attribute)
- *
+ * With no argument the setpoint will be written to the attribute during initialisation!
  */
  	void set_memorized();
+/**
+ * Set the initialisation flag for memorized attributes
+ * 	true  = the setpoint value will be written to the attribute on initialisation
+ *    false = only the attribute setpoint is initialised. No action is taken on the attribute
+ * 
+ * @param	write_on_init	If true the setpoint value will be written to the attribute on initialisation
+ */
+ 	void set_memorized_init(bool write_on_init) {mem_init = write_on_init;}	
+/**
+ * Set a flag to indicate that the server fires change events manually without
+ * the polling to be started for the attribute.
+ * If the detect parameter is set to true, the criteria specified for the change 
+ * event are verified and the event is only pushed if they are fullfilled.
+ * If detect is set to false the event is fired without checking!
+ *
+ * @param implemented True when the server fires change events manually. 
+ * @param detect Triggers the verification of the change event properties when set to true.
+ */
+	void set_change_event(bool implemented, bool detect) 
+			{ ext->fire_change_event  = implemented;
+			  ext->check_change_event = detect; }	
+/**
+ * Check if the change event is fired manually for this attribute.
+ *
+ * @return A boolean set to true if a manual fire change event is implemented.
+ */			
+	bool is_change_event() {return ext->fire_change_event;}
+/**
+ * Check if the change event criteria should be checked when firing
+ * the event manually.
+ *
+ * @return A boolean set to true if a change event criteria will be checked.
+ */			
+	bool is_check_change_criteria() {return ext->check_change_event;}
+/**
+ * Set a flag to indicate that the server fires archive events manually without
+ * the polling to be started for the attribute
+ * If the detect parameter is set to true, the criteria specified for the archive 
+ * event are verified and the event is only pushed if they are fullfilled.
+ * If detect is set to false the event is fired without checking!
+ *
+ * @param implemented True when the server fires archive events manually.
+ * @param detect Triggers the verification of the archive event properties when set to true.
+ */
+	void set_archive_event(bool implemented, bool detect) 
+			{ext->fire_archive_event  = implemented;
+			 ext->check_archive_event = detect;}	
+
+/**
+ * Check if the archive event is fired manually for this attribute.
+ *
+ * @return A boolean set to true if a manual fire archive event is implemented.
+ */			
+	bool is_archive_event() {return ext->fire_archive_event;}	
+/**
+ * Check if the archive event criteria should be checked when firing
+ * the event manually.
+ *
+ * @return A boolean set to true if a archive event criteria will be checked.
+ */			
+	bool is_check_archive_criteria() {return ext->check_archive_event;}
 //@}
 
 	string 			&get_name() {return name;}
@@ -547,6 +636,7 @@ public:
 	Tango::DispLevel	get_disp_level() {return ext->disp_level;}
 	long			get_polling_period() {return ext->poll_period;}
 	bool			get_memorized() {return mem;}
+	bool			get_memorized_init() {return mem_init;}
 	string			&get_assoc() {return assoc_name;}
 	bool			is_assoc()
 				{
@@ -580,6 +670,7 @@ protected:
 	long			type;
 	string			assoc_name;
 	bool			mem;
+	bool			mem_init;		
 	
 	vector<AttrProperty>	class_properties;
 	vector<AttrProperty>	user_default_properties;
@@ -599,8 +690,8 @@ class SpectrumAttrExt
  * database allows the Tango core software to create the Attribute object
  * for the attribute created by the user.
  *
- * $Author: taurel $
- * $Revision: 3.5 $
+ * $Author: jensmeyer $
+ * $Revision: 3.10 $
  */
  
 class SpectrumAttr: public Attr
@@ -689,8 +780,8 @@ class ImageAttrExt
  * database allows the Tango core software to create the Attribute object
  * for the attribute created by the user.
  *
- * $Author: taurel $
- * $Revision: 3.5 $
+ * $Author: jensmeyer $
+ * $Revision: 3.10 $
  */
  
 class ImageAttr: public SpectrumAttr
