@@ -1,4 +1,4 @@
-static const char *RcsId = "$Id: blackbox.cpp,v 3.9 2005/01/13 08:27:32 taurel Exp $\n$Name:  $";
+static const char *RcsId = "$Id: blackbox.cpp,v 3.10 2006/05/18 08:52:37 taurel Exp $\n$Name:  $";
 
 //+============================================================================
 //
@@ -16,9 +16,15 @@ static const char *RcsId = "$Id: blackbox.cpp,v 3.9 2005/01/13 08:27:32 taurel E
 //
 // author(s) :          A.Gotz + E.Taurel
 //
-// $Revision: 3.9 $
+// $Revision: 3.10 $
 //
 // $Log: blackbox.cpp,v $
+// Revision 3.10  2006/05/18 08:52:37  taurel
+// - Miscellaneous changes due to Python device server ported to Windows
+// - Fix some bugs discovered by Windows VC8 using the test suite
+// - Update Windows resource file include path
+// - Fix some Windows VC8 warnings
+//
 // Revision 3.9  2005/01/13 08:27:32  taurel
 // - Merge trunk with Release_5_0 from brach Release_5_branch
 //
@@ -1212,7 +1218,13 @@ void BlackBox::date_ux_to_str(timeval &ux_date,char *str_date)
 
 /* Convert UNIX date to a string in UNIX format */
 
+#ifdef WIN32_VC8
+	time_t vc8_time;
+	vc8_time = (time_t)ux_date.tv_sec;
+	ux_str = ctime(&vc8_time);
+#else
 	ux_str = ctime((time_t *)&(ux_date.tv_sec));
+#endif
 	strcpy(unix_date,ux_str);
 
 /* Copy day */
