@@ -1,4 +1,4 @@
-static const char *RcsId = "$Id: device_2.cpp,v 3.9 2005/05/10 11:26:40 taurel Exp $\n$Name:  $";
+static const char *RcsId = "$Id: device_2.cpp,v 3.12 2006/06/20 08:00:04 jensmeyer Exp $\n$Name:  $";
 
 //+============================================================================
 //
@@ -15,9 +15,21 @@ static const char *RcsId = "$Id: device_2.cpp,v 3.9 2005/05/10 11:26:40 taurel E
 //
 // author(s) :          E.Taurel
 //
-// $Revision: 3.9 $
+// $Revision: 3.12 $
 //
 // $Log: device_2.cpp,v $
+// Revision 3.12  2006/06/20 08:00:04  jensmeyer
+// After changing the the init value of the attribute polling period to -1 insted of 0 I detected
+// a campatibility problem with the old versions. I changed back the init value to 0 and
+// solved the problem with new methods on the Attribute object.
+//
+// Revision 3.11  2006/06/14 11:21:35  jensmeyer
+// Modifications to make the set_polling_period() for attributes work
+// with a polling period set to 0.
+//
+// Revision 3.10  2006/01/20 08:22:29  taurel
+// - Added necessary changes to support Device server written in Python
+//
 // Revision 3.9  2005/05/10 11:26:40  taurel
 // - Fix some deadlock than for attribute for command
 // - Uncomment the two wizard admin device commands
