@@ -1,4 +1,4 @@
-static const char *RcsId = "$Id: dserver.cpp,v 3.13 2005/06/29 08:31:17 taurel Exp $\n$Name:  $";
+static const char *RcsId = "$Id: dserver.cpp,v 3.21 2006/06/28 09:21:28 taurel Exp $\n$Name:  $";
 
 //+=============================================================================
 //
@@ -14,9 +14,39 @@ static const char *RcsId = "$Id: dserver.cpp,v 3.13 2005/06/29 08:31:17 taurel E
 //
 // author(s) :          A.Gotz + E.Taurel
 //
-// $Revision: 3.13 $
+// $Revision: 3.21 $
 //
 // $Log: dserver.cpp,v $
+// Revision 3.21  2006/06/28 09:21:28  taurel
+// - Remove a 1 second sleep at the beginning of the thread executing the Kill command
+//
+// Revision 3.20  2006/06/15 07:08:00  taurel
+// - Fix bug in the order devices/classes are destroyed in the admin device Init command
+//
+// Revision 3.19  2006/05/18 08:52:37  taurel
+// - Miscellaneous changes due to Python device server ported to Windows
+// - Fix some bugs discovered by Windows VC8 using the test suite
+// - Update Windows resource file include path
+// - Fix some Windows VC8 warnings
+//
+// Revision 3.18  2006/04/06 09:57:59  taurel
+// - Minor changes in error message in the create_cpp_class method
+//
+// Revision 3.17  2006/03/09 08:18:08  taurel
+// - Change the order of device server destruction. First devices then class.
+// It was the contrary....
+//
+// Revision 3.16  2006/02/17 16:51:28  jensmeyer
+// Corrections when porting to VC7 under windows
+//
+// Revision 3.15  2006/01/27 14:27:10  taurel
+// - Fix a severe incompatibility problem introduced by all the modifs done for
+// PY DS
+// - Duplicate some EventSupplier class methods (instead of using template) in order to be able to generate Tango shared library on Suse 9.3
+//
+// Revision 3.14  2006/01/20 08:22:29  taurel
+// - Added necessary changes to support Device server written in Python
+//
 // Revision 3.13  2005/06/29 08:31:17  taurel
 // - Last commit before release 5.2 ?
 //
@@ -261,13 +291,17 @@ static const char *RcsId = "$Id: dserver.cpp,v 3.13 2005/06/29 08:31:17 taurel E
 #ifndef WIN32
 #include <unistd.h>
 #include <signal.h>
+#include <dlfcn.h>
 #endif /* WIN32 */
 
 #include <stdlib.h>
 
+extern omni_thread::key_t key_py_data;
 namespace Tango
 {
 
+
+
 //+----------------------------------------------------------------------------
 //
 // method : 		DServer::DServer(string &s)
@@ -311,21 +345,22 @@ bool less_than (Command *a,Command *b)
 
 void DServer::init_device()
 {
+
+//
 // In case of database 
-	{
-		Tango::Util *tg = Tango::Util::instance();
-		if (tg->_FileDb)
-			tg->reset_filedatabase();
-	}
+//
+
+	Tango::Util *tg = Tango::Util::instance();
+	if (tg->_FileDb)
+		tg->reset_filedatabase();
+	
 //	
 // Destroy already registered classes
 //
 
 	if (class_list.empty() == false)
 	{
-		for (unsigned long i = 0;i < class_list.size();i++)
-			delete class_list[i];
-		class_list.clear();
+		delete_devices();
 	}
 		
 	cout3 << "DServer::DSserver() create dserver " << device_name << endl;
@@ -343,7 +378,7 @@ void DServer::init_device()
 		PortableServer::POAManager_var manager = poa->the_POAManager();
 
 		PortableServer::POAManager::State man_state = manager->get_state();
-		if (man_state == PortableServer::POAManager::HOLDING)	
+		if ((man_state == PortableServer::POAManager::HOLDING) || (man_state == PortableServer::POAManager::DISCARDING))	
 			manager->activate();
 						
 //
@@ -360,7 +395,6 @@ void DServer::init_device()
 // Set the class list pointer in the Util class and add the DServer object class
 //
 
-			Tango::Util *tg = Tango::Util::instance();
 			tg->set_class_list(&class_list);
 			tg->add_class_to_list(this->get_device_class());
 
@@ -416,7 +450,7 @@ void DServer::init_device()
 				                			(const char *)"Dserver::init_device");			
 					}				
 					
-					long nb_dev = na.size();				
+					long nb_dev = na.size();
 					Tango::DevVarStringArray dev_list(nb_dev);
 					dev_list.length(nb_dev);
 					
@@ -437,9 +471,30 @@ void DServer::init_device()
 //
 // Set value for each device with memorized writable attr
 // This is necessary only if db is used
+// For Python device server, writing the attribute will tak the Python lock.
+// If we already have it --> dead lock.
+// Release the python lock if we already have it before calling the set_memorized_values
+// method
 //
+					
+					PyLock *lock_ptr;
+					omni_thread *th;
+					
+					if (tg->is_py_ds() == true)
+					{
+						th = omni_thread::self();	
 
+						omni_thread::value_t *tmp_py_data = th->get_value(key_py_data);
+						lock_ptr = (static_cast<PyData *>(tmp_py_data))->PerTh_py_lock;
+						lock_ptr->Release();
+					}
+					
 					class_list[i]->set_memorized_values(true);
+					
+					if (tg->is_py_ds() == true)
+					{
+						lock_ptr->Get();
+					}
 				}
 				else
 				{
@@ -485,7 +540,6 @@ void DServer::init_device()
 	}
 	catch (bad_alloc)
 	{
-		
 //
 // If the class_factory method have not been successfully executed, erase
 // all classes already built. If the error occurs during the command or device
@@ -500,8 +554,14 @@ void DServer::init_device()
 			o << "Can't allocate memory in server while creating class number " << class_err << ends;
 			if (class_list.empty() == false)
 			{
+
 				for (unsigned long j = 0;j < class_list.size();j++)
-					delete class_list[j];
+				{
+					if (class_list[i]->is_py_class() == false)
+						delete class_list[i];
+					else
+						class_list[i]->delete_class();			
+				}
 				class_list.clear();
 			}
 		}
@@ -509,7 +569,12 @@ void DServer::init_device()
 		{
 			o << "Can't allocate memory in server while building command(s) or device(s) for class number " << i + 1 << ends;
 			for (unsigned long j = i;j < class_list.size();j++)
-				delete class_list[j];
+			{
+				if (class_list[i]->is_py_class() == false)
+					delete class_list[i];
+				else
+					class_list[i]->delete_class();	
+			}
 			class_list.erase(class_list.begin() + i,class_list.end());
 		}
 		
@@ -517,8 +582,8 @@ void DServer::init_device()
 				      o.str(),
 				      (const char *)"DServer::init_device");
 	}
-	catch (Tango::NamedDevFailedList)
-	{
+	catch (Tango::NamedDevFailedList &)
+	{	
 	
 //
 // If the class_factory method have not been successfully executed, erase
@@ -531,14 +596,24 @@ void DServer::init_device()
 			if (class_list.empty() == false)
 			{
 				for (unsigned long j = 0;j < class_list.size();j++)
-					delete class_list[j];
+				{
+					if (class_list[i]->is_py_class() == false)
+						delete class_list[i];
+					else
+						class_list[i]->delete_class();					
+				}
 				class_list.clear();
 			}
 		}
 		else
 		{
 			for (unsigned long j = i;j < class_list.size();j++)
-				delete class_list[j];
+			{
+				if (class_list[i]->is_py_class() == false)
+					delete class_list[i];
+				else
+					class_list[i]->delete_class();	
+			}
 			class_list.erase(class_list.begin() + i,class_list.end());
 		}
 		throw;
@@ -557,14 +632,25 @@ void DServer::init_device()
 			if (class_list.empty() == false)
 			{
 				for (unsigned long j = 0;j < class_list.size();j++)
-					delete class_list[j];
+				{
+					if (class_list[j]->is_py_class() == false)
+						delete class_list[j];
+					else
+					{
+						class_list[j]->delete_class();
+						break;
+					}
+				}
 				class_list.clear();
 			}
 		}
 		else
 		{
 			for (unsigned long j = i;j < class_list.size();j++)
-				delete class_list[j];
+			{
+				if (class_list[j]->is_py_class() == false)
+					delete class_list[j];
+			}
 			class_list.erase(class_list.begin() + i,class_list.end());
 		}
 		throw;
@@ -586,6 +672,7 @@ DServer::~DServer()
 //	
 // Destroy already registered classes
 //
+
 	{
 		Tango::Util *tg = Tango::Util::instance();
 		if (tg->_FileDb)
@@ -595,7 +682,15 @@ DServer::~DServer()
 	if (class_list.empty() == false)
 	{
 		for (long i = class_list.size() - 1;i >= 0;i--)
-			delete class_list[i];
+		{
+			if (class_list[i]->is_py_class() == false)
+				delete class_list[i];
+			else
+			{
+				class_list[i]->delete_class();
+				break;
+			}	
+		}
 		class_list.clear();
 	}
 	
@@ -617,7 +712,29 @@ void DServer::delete_devices()
 	if (class_list.empty() == false)
 	{
 		for (long i = class_list.size() - 1;i >= 0;i--)
-			delete class_list[i];
+		{
+			if (class_list[i]->is_py_class() == false)
+			{
+				Tango::Util *tg = Tango::Util::instance();
+				PortableServer::POA_ptr r_poa = tg->get_poa();
+				unsigned long loop;
+
+				vector<DeviceImpl *> &devs = class_list[i]->get_device_list();
+				for (loop = 0;loop < devs.size();loop++)
+				{	
+					class_list[i]->delete_dev(loop,tg,r_poa);		
+				}
+				devs.clear();
+				CORBA::release(r_poa);
+				
+				delete class_list[i];
+			}
+			else
+			{
+				class_list[i]->delete_class();
+				break;
+			}
+		}
 		class_list.clear();
 	}
 
@@ -850,7 +967,15 @@ void DServer::restart(string &d_name)
 	PortableServer::POA_ptr r_poa = tg->get_poa();
 	if (dev_to_del->get_exported_flag() == true)
 		r_poa->deactivate_object(dev_to_del->get_obj_id().in());
-	dev_to_del->_remove_ref();
+	if (dev_to_del->is_py_device() == false)
+		dev_to_del->_remove_ref();
+	else
+	{
+		AutoPyLock PyLo;
+		
+		Device_3Impl *dev_to_del_3 = static_cast<Device_3Impl *>(dev_to_del);		
+		dev_to_del_3->delete_dev();
+	}	
 	CORBA::release(r_poa);
 		
 //
@@ -859,7 +984,9 @@ void DServer::restart(string &d_name)
 //
 
 	{
-		AutoTangoMonitor sync(dev_cl);	
+		AutoTangoMonitor sync(dev_cl);
+		AutoPyLock PyLo;		
+	
 		dev_cl->device_factory(&name);
 	}
 
@@ -973,8 +1100,11 @@ void DServer::restart_server()
 	
 }
 
-void *ServRestartThread::run_undetached(void *ptr)
+
+void ServRestartThread::run(void *ptr)
 {
+	PyData *py_data_ptr = new PyData();
+	omni_thread::self()->set_value(key_py_data,py_data_ptr);
 
 //
 // The arg. passed to this method is a pointer to the DServer device
@@ -1010,7 +1140,7 @@ void *ServRestartThread::run_undetached(void *ptr)
 //
 // Destroy and recreate the multi attribute object
 //
-	
+
 	MultiAttribute *tmp_ptr;
 	try
 	{
@@ -1025,24 +1155,29 @@ void *ServRestartThread::run_undetached(void *ptr)
 	dev->add_state_status_attrs();
 				
 //	
-// Restart the device
+// Restart device(s)
 //
 
-	dev->init_device();
+	{
+		AutoPyLock PyLo;
+		
+		dev->init_device();
+	}
 	
 //
 // Restart polling (if any)
 //
 
 	Tango::Util *tg = Tango::Util::instance();
-
 	tg->polling_configure();
 
 //
-// Just make some compiler happy
+// Exit thread
 //
 
-  	return 0;
+	omni_thread::self()->remove_value(key_py_data);
+	delete py_data_ptr;
+	omni_thread::exit();
 }
 
 
@@ -1218,7 +1353,7 @@ void *KillThread::run_undetached(void *ptr)
 {
 	cout4 << "In the killer thread !!!" << endl;
 	
-	sleep(1);
+	omni_thread::self()->set_value(key_py_data,new PyData());
 
 //
 //  Send kill command to the polling thread
@@ -1250,7 +1385,12 @@ void *KillThread::run_undetached(void *ptr)
 // Delete devices (except the admin one)
 //
 
-	tg->get_dserver_device()->delete_devices();
+
+	{
+		AutoPyLock PyLo;
+		tg->get_dserver_device()->delete_devices();
+	}
+	
 
 //
 // Write file database for server started with the -file option
@@ -1281,5 +1421,105 @@ void *KillThread::run_undetached(void *ptr)
 	return NULL;
 }
 
+
+//+----------------------------------------------------------------------------
+//
+// method : 		DServer::create_cpp_class()
+// 
+// description : 	Create a Cpp Tango class from its name
+//
+//-----------------------------------------------------------------------------
+
+
+void DServer::create_cpp_class(const char *cl_name,const char *par_name)
+{
+	cout4 << "In DServer::create_cpp_class for " << cl_name <<  ", " << par_name << endl;
+	string class_name(cl_name);
+	string lib_name = class_name;
+	
+#ifdef WIN32
+	HMODULE mod;
+
+	if ((mod = LoadLibrary(lib_name.c_str())) == NULL)
+	{
+   		char *str = 0;
+		
+		DWORD l_err = GetLastError();
+    		::FormatMessage(FORMAT_MESSAGE_ALLOCATE_BUFFER | FORMAT_MESSAGE_FROM_SYSTEM,NULL,
+			      l_err,MAKELANGID(LANG_NEUTRAL, SUBLANG_DEFAULT),(char *)&str,0,NULL);
+
+		cerr << "Error: " << str << endl;
+
+		TangoSys_OMemStream o;
+		o << "Trying to load shared library " << lib_name << " failed. It returns error: " << str << ends;
+		::LocalFree((HLOCAL)str);
+
+		Except::throw_exception((const char *)"API_ClassNotFound",o.str(),
+				        (const char *)"DServer::create_cpp_class");
+	}
+	
+	cout4 << "GetModuleHandle is a success" << endl;
+	
+	string sym_name("_create_");
+	sym_name = sym_name + class_name;
+	sym_name = sym_name + "_class";
+	
+	cout4 << "Symbol name = " << sym_name << endl;
+	FARPROC proc;
+	
+	if ((proc = GetProcAddress(mod,sym_name.c_str())) == NULL)
+	{
+		TangoSys_OMemStream o;
+		o << "Class " << cl_name << " does not have the C creator function (_create_<Class name>_class)" << ends;
+
+		Except::throw_exception((const char *)"API_ClassNotFound",o.str(),
+				        (const char *)"DServer::create_cpp_class");
+	}
+	cout4 << "GetProcAddress is a success" << endl;
+	
+	Cpp_creator_ptr mt = (Cpp_creator_ptr)proc;
+#else
+	void *lib_ptr;
+	lib_name = lib_name + ".so";
+	
+	lib_ptr = dlopen(lib_name.c_str(),RTLD_NOW);
+	if (lib_ptr == NULL)
+	{
+		TangoSys_OMemStream o;
+		o << "Trying to load shared library " << lib_name << " failed. It returns error: " << dlerror() << ends;
+
+		Except::throw_exception((const char *)"API_ClassNotFound",o.str(),
+				        (const char *)"DServer::create_cpp_class");
+	}
+	
+	cout4 << "dlopen is a success" << endl;
+	
+	void *sym;
+
+	string sym_name("_create_");
+	sym_name = sym_name + class_name;
+	sym_name = sym_name + "_class";
+	
+	cout4 << "Symbol name = " << sym_name << endl;
+		
+	sym = dlsym(lib_ptr,sym_name.c_str());
+	if (sym == NULL)
+	{
+		TangoSys_OMemStream o;
+		o << "Class " << cl_name << " does not have the C creator function (_create_<Class name>_class)" << ends;
+
+		Except::throw_exception((const char *)"API_ClassNotFound",o.str(),
+				        (const char *)"DServer::create_cpp_class");
+	}
+	
+	cout4 << "dlsym is a success" << endl;
+	
+	Cpp_creator_ptr mt = (Cpp_creator_ptr)sym;
+#endif /* WIN32 */
+	Tango::DeviceClass *dc = (*mt)(par_name);
+	add_class(dc);
+}
+
+
 } // End of Tango namespace
 
