@@ -1,4 +1,4 @@
-static const char *RcsId = "$Id: event.cpp,v 1.13 2006/06/20 15:17:55 jensmeyer Exp $";
+static const char *RcsId = "$Id: event.cpp,v 1.14 2006/09/20 07:46:57 jensmeyer Exp $";
 
 ////////////////////////////////////////////////////////////////////////////////
 ///
@@ -14,9 +14,12 @@ static const char *RcsId = "$Id: event.cpp,v 1.13 2006/06/20 15:17:55 jensmeyer
 ///
 ///		original : 7 April 2003
 ///
-///		$Revision: 1.13 $
+///		$Revision: 1.14 $
 ///
 ///		$Log: event.cpp,v $
+///		Revision 1.14  2006/09/20 07:46:57  jensmeyer
+///		Corrected reconnection to notify daemon in case of network connection problems
+///		
 ///		Revision 1.13  2006/06/20 15:17:55  jensmeyer
 ///		Corrected bug in the event reconnection.
 ///		The bug was introduced when implementing the host name check
@@ -595,6 +598,7 @@ void EventConsumer::connect_event_channel(string &channel_name,Database *db,bool
 		channel_map[channel_name].last_heartbeat = time(NULL);
 		channel_map[channel_name].heartbeat_skipped = false;
 		channel_map[channel_name].notifyd_host = hostname;
+		channel_map[channel_name].notifd_failed = false;
 	}
 	else
 	{
@@ -605,6 +609,7 @@ void EventConsumer::connect_event_channel(string &channel_name,Database *db,bool
 		new_event_channel_struct.adm_device_proxy = NULL;
 		channel_map[channel_name] = new_event_channel_struct;
 		channel_map[channel_name].notifyd_host = hostname;
+		channel_map[channel_name].notifd_failed = false;
 	}
 
 //
@@ -1826,12 +1831,15 @@ void *EventConsumerKeepAliveThread::run_undetached(void *arg)
 // If a heartbeat is missing, there are two possibilities :
 // 1 - The notifd is dead (or the crate is rebooting or has already reboot)
 // 2 - The server is dead
+// 3- The network was down;
+// 4- The server has been restarted on another host.
 //
 
  			bool heartbeat_skipped;
 
 			heartbeat_skipped = ((now - ipos->second.last_heartbeat) > (EVENT_HEARTBEAT_PERIOD + 1));
-			if (heartbeat_skipped || ipos->second.heartbeat_skipped)
+			if (heartbeat_skipped || ipos->second.heartbeat_skipped ||
+			   ipos->second.notifd_failed == true )
 			{
 				ipos->second.heartbeat_skipped = true;
 				
@@ -1839,7 +1847,7 @@ void *EventConsumerKeepAliveThread::run_undetached(void *arg)
 // Check notifd by trying to read an attribute of the event channel
 //
 
-				bool notifd_failed = false;
+				//bool notifd_failed = false;
 				try
 				{
 // 
@@ -1861,14 +1869,14 @@ void *EventConsumerKeepAliveThread::run_undetached(void *arg)
 					//cout << "New host = " << info.server_host << endl;
 					if ( ipos->second.notifyd_host != info.server_host )
 						{
-						notifd_failed = true;
+						ipos->second.notifd_failed = true;
 						}
 					else
 						CosNotifyChannelAdmin::EventChannelFactory_var ecf = ipos->second.eventChannel->MyFactory();
 				}
 				catch (...)
 				{
-					notifd_failed = true;
+					ipos->second.notifd_failed = true;
 					cout3 << "Notifd is dead !!!" << endl;
 				}
 
@@ -1879,12 +1887,17 @@ void *EventConsumerKeepAliveThread::run_undetached(void *arg)
 // callbacks to this new event channel
 //
 
-				bool notifd_reco = true;
-				if (notifd_failed == true)
+				//cout << "Notify daemon failed : " << ipos->second.notifd_failed << endl;
+				if ( ipos->second.notifd_failed == true )
 				{
-					notifd_reco = reconnect_to_channel(ipos,event_consumer);
+					bool notifd_reco = reconnect_to_channel(ipos,event_consumer);
+					//cout << "Notify daemon reconnect : " << notifd_reco << endl;
+					if ( notifd_reco )
+						ipos->second.notifd_failed = false;
+					else
+						ipos->second.notifd_failed = true;
 
-					if (notifd_reco == true)
+					if ( ipos->second.notifd_failed == false ) 
 					{
 						reconnect_to_event(ipos,event_consumer);
 					}
@@ -1939,7 +1952,7 @@ void *EventConsumerKeepAliveThread::run_undetached(void *arg)
 						callback->push_event(event_data);
 						delete event_data;
 						
-       						if (notifd_reco == true)
+						if ( ipos->second.notifd_failed == false )
 						{
 							DeviceData subscriber_in;
        							vector<string> subscriber_info;
