@@ -12,7 +12,7 @@
 ///
 /// 		original : 7 April 2003
 ///
-/// 		$Revision: 1.7 $
+/// 		$Revision: 1.13 $
 ///
 /// 		copyright : European Synchrotron Radiation Facility
 ///                         BP 220, Grenoble 38043
@@ -42,7 +42,7 @@
 #ifndef WIN32
 #include <sys/time.h>
 #endif
-
+#include <math.h>
 
 
 namespace Tango
@@ -109,7 +109,49 @@ private :
 	TANGO_IMP_EXP static void connect_to_notifd(NotifService &,
 			CORBA::ORB_var &,string &,Database *,string &);
 			
+	// Added a mutex to synchronize the access to 
+	//	detect_and_push_change_event_3	and
+	// detect_and_push_archive_event_3 which are used
+	// from different threads
+	omni_mutex		event_mutex;
+	
+	// Added a mutex to synchronize the access to 
+	//	push_event_3 which is used
+	// from different threads
+	omni_mutex		push_mutex;
+	
+	// Added a mutex to synchronize the access to 
+	//	detect_event_3 which is used
+	// from different threads
+	omni_mutex		detect_mutex;
+	
 public :
+	void detect_and_push_events_3(DeviceImpl *,long,AttributeValue_3 &,DevFailed *,string &);
+
+
+
+/////////////////////////////////////////////////////////////////////////////
+//
+//			WARNING
+//			-------
+//
+////////////////////////////////////////////////////////////////////////////
+
+// Unfortunately, with Suse 9.3 with gcc 3.3.4 and binutils 2.15.94,
+// it was not possible to generate the shared libs of Tango
+// (linker error messages).
+// To solve this problem, it was necessary to duplicate these methods
+// instead of simply using a template method. This generates a code
+// duplicate but...... (all methods finishing with xxxx_3)
+//
+// Test with gcc 3.4.5 with binutils 2.16.1 did not solve the problems
+// We need to check with gcc 4.x.x if we could remove this code duplicate
+// The other methods are in eventsupplier.cpp
+//
+//
+//--------------------------------------------------------------------------
+
+				    
 	template <typename T>
 	void detect_and_push_events(DeviceImpl *device_impl,
 				    long idl_vers,
@@ -118,7 +160,7 @@ public :
 				    string &attr_name)
 	{
 		string event, domain_name;
-		int now, change_subscription, quality_subscription, periodic_subscription, archive_subscription;
+		int now, change_subscription, periodic_subscription, archive_subscription;
 
 		cout3 << "EventSupplier::detect_and_push_events(): called for attribute " << attr_name << endl;
 
@@ -126,19 +168,19 @@ public :
 
 		now = time(NULL);
 		change_subscription = now - attr.ext->event_change_subscription;
-		quality_subscription = now - attr.ext->event_quality_subscription;
+		//quality_subscription = now - attr.ext->event_quality_subscription;
 		periodic_subscription = now - attr.ext->event_periodic_subscription;
 		archive_subscription = now - attr.ext->event_archive_subscription;
 
-		cout3 << "EventSupplier::detect_and_push_events(): last subscription for change " << change_subscription << " quality " << quality_subscription << " periodic " << periodic_subscription << " archive " << archive_subscription << endl;
+		cout3 << "EventSupplier::detect_and_push_events(): last subscription for change " << change_subscription << " periodic " << periodic_subscription << " archive " << archive_subscription << endl;
 		if (change_subscription < EVENT_RESUBSCRIBE_PERIOD)
 		{
 			detect_and_push_change_event(device_impl,attr_value,attr,attr_name,except);
 		}
-		if (quality_subscription < EVENT_RESUBSCRIBE_PERIOD)
-		{
-			detect_and_push_quality_change_event(device_impl,attr_value,attr,attr_name,except);
-		}
+		//if (quality_subscription < EVENT_RESUBSCRIBE_PERIOD)
+		//{
+		//	detect_and_push_quality_change_event(device_impl,attr_value,attr,attr_name,except);
+		//}
 		if (periodic_subscription < EVENT_RESUBSCRIBE_PERIOD)
 		{
 			detect_and_push_periodic_event(device_impl,attr_value,attr,attr_name,except);
@@ -173,6 +215,17 @@ public :
 //
 //-----------------------------------------------------------------------------
 
+
+	bool detect_change_3(Attribute &attr,
+				  AttributeValue_3 &curr_attr_value,
+				  bool archive,
+				  double &delta_change_rel,
+				  double &delta_change_abs,
+				  DevFailed *except,
+				  bool &force_change,
+				  DeviceImpl *dev);
+				  
+				  
 	template <typename T>
 	bool detect_change(Attribute &attr,
 				  T &curr_attr_value,
@@ -184,7 +237,12 @@ public :
 				  DeviceImpl *dev)
 	{
 		bool is_change = false;
-
+		
+		cout3 << "EventSupplier::detect_change(): called for attribute " << attr.get_name() << endl;
+		
+		// get the mutex to synchronize the sending of events
+		omni_mutex_lock l(detect_mutex);
+		
 //
 // Send event, if the read_attribute failed or if it is the first time
 // that the read_attribute succeed after a failure.
@@ -281,7 +339,7 @@ public :
 		mon1.rel_monitor();
 
 		if (inited)
-		{		
+		{	
 			if (enable_check == true)
 			{
 				if (ty->kind() == CORBA::tk_enum)
@@ -303,7 +361,8 @@ public :
 				{
 				
 					CORBA::TypeCode_var ty_alias = ty->content_type();
-					CORBA::TypeCode_var ty_seq = ty_alias->content_type();					switch (ty_seq->kind())
+					CORBA::TypeCode_var ty_seq = ty_alias->content_type();					
+					switch (ty_seq->kind())
 					{
 					case CORBA::tk_long:		
 						curr_attr_value.value >>= curr_seq_lo;
@@ -465,6 +524,7 @@ public :
 									delta_change_rel = 100;
 									if ((*curr_seq_db)[i] == (*prev_seq_db)[i]) delta_change_rel = 0;
 								}
+								
 								if (delta_change_rel <= rel_change[0] || delta_change_rel >= rel_change[1])
 								{
 									is_change = true;
@@ -474,7 +534,13 @@ public :
 							if (abs_change[0] != INT_MAX)
 							{
 								delta_change_abs = (*curr_seq_db)[i] - (*prev_seq_db)[i];
-								if (delta_change_abs <= abs_change[0] || delta_change_abs >= abs_change[1])
+								
+								// Correct for rounding errors !
+								double max_change = delta_change_abs + (abs_change[1] * 1e-10);
+								double min_change = delta_change_abs + (abs_change[0] * 1e-10);
+								
+								//if (delta_change_abs <= abs_change[0] || delta_change_abs >= abs_change[1])
+								if (min_change <= abs_change[0] || max_change >= abs_change[1])
 								{
 									is_change = true;
 									return(is_change);
@@ -542,7 +608,13 @@ public :
 							if (abs_change[0] != INT_MAX)
 							{
 								delta_change_abs = (*curr_seq_fl)[i] - (*prev_seq_fl)[i];
-								if (delta_change_abs <= abs_change[0] || delta_change_abs >= abs_change[1])
+								
+								// Correct for rounding errors !
+								double max_change = delta_change_abs + (abs_change[1] * 1e-10);
+								double min_change = delta_change_abs + (abs_change[0] * 1e-10);
+																
+								//if (delta_change_abs <= abs_change[0] || delta_change_abs >= abs_change[1])
+								if (min_change <= abs_change[0] || max_change >= abs_change[1])
 								{
 									is_change = true;
 									return(is_change);
@@ -667,9 +739,17 @@ public :
 			}
 		}	
 
+		cout3 << "EventSupplier::detect_change(): leaving for attribute " << attr.get_name() << endl;
 		return(is_change);
 	}
 
+
+	void detect_and_push_change_event_3(DeviceImpl *device_impl,
+						 AttributeValue_3 &attr_value,
+						 Attribute &attr,
+						 string &attr_name,
+						 DevFailed *except);
+	
 	template <typename T>
 	void detect_and_push_change_event(DeviceImpl *device_impl,
 						 T &attr_value,
@@ -680,11 +760,15 @@ public :
 		string event, domain_name;
 		double delta_change_rel = 0.0;
 		double delta_change_abs = 0.0;
-		bool is_change = false;
-		bool force_change = false;
+		bool is_change      = false;
+		bool force_change   = false;
+		bool quality_change = false;
 
 		cout3 << "EventSupplier::detect_and_push_change_event(): called for attribute " << attr_name << endl;
 
+		// get the mutex to synchronize the sending of events
+		omni_mutex_lock l(event_mutex);
+
 //
 // if no attribute of this name is registered with change then
 // insert the current value
@@ -708,10 +792,19 @@ public :
 //
 // determine delta_change in percent compared with previous event sent
 // 
-
 			is_change = detect_change(attr,attr_value,false,delta_change_rel,delta_change_abs,except,force_change,device_impl);
 			cout3 << "EventSupplier::detect_and_push_change_event(): rel_change " << delta_change_rel << " abs_change " << delta_change_abs << " is change = " << is_change << endl;
 		}
+		
+		// check whether the data quality has changed.
+		// Fire event on a quality change.
+		
+		if ( attr.ext->prev_change_event.quality != attr_value.quality )
+			{
+			is_change = true;
+			quality_change = true;
+			}
+		
 	
 		if (is_change)
 		{
@@ -734,11 +827,19 @@ public :
 			filterable_data.push_back(delta_change_rel);
 			filterable_names.push_back("delta_change_abs");
 			filterable_data.push_back(delta_change_abs);
+			
 			filterable_names.push_back("forced_event");
 			if (force_change == true)
 				filterable_data.push_back((double)1.0);
 			else
 				filterable_data.push_back((double)0.0);
+			
+			filterable_names.push_back("quality");
+			if (quality_change == true)
+				filterable_data.push_back((double)1.0);
+			else
+				filterable_data.push_back((double)0.0);				
+			
 			push_event(device_impl,
 				   "change",
 				   filterable_names,
@@ -749,6 +850,7 @@ public :
 				   attr_name,
 				   except);
 		}
+	cout3 << "EventSupplier::detect_and_push_change_event(): leaving for attribute " << attr_name << endl;		
 	}
 
 //+----------------------------------------------------------------------------
@@ -767,6 +869,12 @@ public :
 //
 //-----------------------------------------------------------------------------
 
+	void detect_and_push_archive_event_3(DeviceImpl *device_impl,
+						  AttributeValue_3 &attr_value, 
+						  Attribute &attr,
+						  string &attr_name,
+						  DevFailed *except);
+						  
 	template <typename T>
 	void detect_and_push_archive_event(DeviceImpl *device_impl,
 						  T &attr_value, 
@@ -797,6 +905,10 @@ public :
 #else
 		gettimeofday(&now_timeval,NULL);
 #endif
+
+		// get the mutex to synchronize the sending of events
+		omni_mutex_lock l(event_mutex);
+		
 		now_ms = (double)now_timeval.tv_sec * 1000. + (double)now_timeval.tv_usec / 1000.;
 		ms_since_last_periodic = now_ms - attr.ext->archive_last_periodic;
 
@@ -805,6 +917,38 @@ public :
 		mon1.get_monitor();
 		arch_period = attr.ext->archive_period;
 		mon1.rel_monitor();
+
+		// Specify the precision interval for the archive period testing
+		// 2% are used for periods < 5000 ms and
+		// 100ms are used for periods > 5000 ms.
+	
+		if ( arch_period >= 5000 )
+			{
+			 arch_period = arch_period - DELTA_PERIODIC_LONG;
+			}
+		else
+			{
+#ifdef WIN32
+			double tmp = (double)arch_period * DELTA_PERIODIC;
+			double int_part,eve_round;
+			double frac = modf(tmp,&int_part);
+			if (frac >= 0.5)
+					eve_round = ceil(tmp);
+			else
+					eve_round = floor(tmp);					
+#else
+	#if ((defined __SUNPRO_CC) || (!defined GCC_STD) || (defined __HP_aCC))
+			double eve_round = rint((double)arch_period * DELTA_PERIODIC);
+	#else
+		#if (defined GCC_SOLARIS)
+			double eve_round = rint((double)arch_period * DELTA_PERIODIC);
+		#else
+			double eve_round = round((double)arch_period * DELTA_PERIODIC);
+		#endif
+	#endif
+#endif
+			arch_period = (int)eve_round;
+			}
 		
 		if ((ms_since_last_periodic > arch_period) && (attr.ext->prev_archive_event.inited == true))
 		{
@@ -857,17 +1001,17 @@ public :
 			vector<long> filterable_data_lg;
 
 			domain_name = device_impl->get_name() + "/" + attr_name;
-			if (value_change == true)
+//			if (value_change == true)
+//			{
+			if (except != NULL)
+				attr.ext->prev_archive_event.err = true;
+			else
 			{
-				if (except != NULL)
-					attr.ext->prev_archive_event.err = true;
-				else
-				{
-					attr.ext->prev_archive_event.value = attr_value.value;
-					attr.ext->prev_archive_event.quality = attr_value.quality;
-					attr.ext->prev_archive_event.err = false;
-				}
+				attr.ext->prev_archive_event.value = attr_value.value;
+				attr.ext->prev_archive_event.quality = attr_value.quality;
+				attr.ext->prev_archive_event.err = false;
 			}
+//			}
 
 			filterable_names_lg.push_back("counter");
 			if (period_change == true)
@@ -920,6 +1064,13 @@ public :
 //
 //-----------------------------------------------------------------------------
 
+	void detect_and_push_quality_change_event_3(DeviceImpl *device_impl,
+						  AttributeValue_3 &attr_value,
+						  Attribute &attr,
+						  string &attr_name,
+						  DevFailed *except);
+
+
 	template <typename T>
 	void detect_and_push_quality_change_event(DeviceImpl *device_impl,
 						  T &attr_value,
@@ -1006,6 +1157,13 @@ public :
 //
 //-----------------------------------------------------------------------------
 
+	void detect_and_push_periodic_event_3(DeviceImpl *device_impl,
+					    AttributeValue_3 &attr_value,
+					    Attribute &attr,
+					    string &attr_name,
+					    DevFailed *except);
+					    
+
 	template <typename T>
 	void detect_and_push_periodic_event(DeviceImpl *device_impl,
 					    T &attr_value,
@@ -1027,6 +1185,10 @@ public :
 #else
 		gettimeofday(&now_timeval,NULL);
 #endif
+
+		// get the mutex to synchronize the sending of events
+		omni_mutex_lock l(event_mutex);
+
 		now_ms = (double)now_timeval.tv_sec * 1000. + (double)now_timeval.tv_usec / 1000.;
 
 		int eve_period;		
@@ -1035,6 +1197,38 @@ public :
 		eve_period = attr.ext->event_period;
 		mon1.rel_monitor();
 
+		// Specify the precision interval for the event period testing
+		// 2% are used for periods < 5000 ms and
+		// 100ms are used for periods > 5000 ms.
+	
+		if ( eve_period >= 5000 )
+			{
+			 eve_period = eve_period - DELTA_PERIODIC_LONG;
+			}
+		else
+			{
+#ifdef WIN32
+			double tmp = (double)eve_period * DELTA_PERIODIC;
+			double int_part,eve_round;
+			double frac = modf(tmp,&int_part);
+			if (frac >= 0.5)
+					eve_round = ceil(tmp);
+			else
+					eve_round = floor(tmp);					
+#else
+	#if ((defined __SUNPRO_CC) || (!defined GCC_STD) || (defined __HP_aCC))
+			double eve_round = rint((double)eve_period * DELTA_PERIODIC);
+	#else
+		#if (defined GCC_SOLARIS)
+			double eve_round = rint((double)eve_period * DELTA_PERIODIC);
+		#else
+			double eve_round = round((double)eve_period * DELTA_PERIODIC);
+		#endif
+	#endif
+#endif
+			eve_period = (int)eve_round;
+			}
+
 		ms_since_last_periodic = now_ms - attr.ext->last_periodic;
 		cout3 << "EventSupplier::detect_and_push_is_periodic_event(): delta since last periodic " << ms_since_last_periodic << " event_period " << eve_period << " for " << device_impl->get_name()+"/"+attr_name << endl;
 
@@ -1081,6 +1275,17 @@ public :
 //
 //-----------------------------------------------------------------------------
 
+	void push_event_3(DeviceImpl *device_impl,
+			string event_type,
+			vector<string> &filterable_names,
+		       	vector<double> &filterable_data,
+		       	vector<string> &filterable_names_lg,
+		       	vector<long> &filterable_data_lg,
+		       	AttributeValue_3 &attr_value,
+		       	string &attr_name,
+		       	DevFailed *except);
+			
+
 	template <typename T>
 	void push_event(DeviceImpl *device_impl,
 			string event_type,
@@ -1096,7 +1301,10 @@ public :
 		string domain_name;
 
 		cout3 << "EventSupplier::push_event(): called for attribute " << attr_name << endl;
-
+		
+		// get the mutex to synchronize the sending of events
+		omni_mutex_lock l(push_mutex);
+		
 		string loc_attr_name = attr_name;	
 		transform(loc_attr_name.begin(),loc_attr_name.end(),loc_attr_name.begin(),::tolower);
 		domain_name = device_impl->get_name_lower() + "/" + loc_attr_name;
@@ -1195,3 +1403,176 @@ public :
 
 
 #endif // _EVENT_SUPPLIER_API_H
+
+
+
+
+
+
+
+
+
+
+
+
+
+
+
+
+
+
+
+
+
+
+
+
+
+
+
+
+
+
+
+
+
+
+
+
+
+
+
+
+
+
+
+
+
+
+
+
+
+
+
+
+
+
+
+
+
+
+
+
+
+
+
+
+
+
+
+
+
+
+
+
+
+
+
+
+
+
+
+
+
+
+
+
+
+
+
+
+
+
+
+
+
+
+
+
+
+
+
+
+
+
+
+
+
+
+
+
+
+
+
+
+
+
+
+
+
+
+
+
+
+
+
+
+
+
+
+
+
+
+
+
+
+
+
+
+
+
+
+
+
+
+
+
+
+
+
+
+
+
+
+
+
+
+
+
+
+
+
+
+
+
+
+
+
+
+
+
+
+
+
+
+
