@@ -437,7 +437,7 @@ stopped_(false)
 
 	char *tmp;
 
-	if ((tmp = strrchr(name,'\\')) == 0)
+	if ((tmp = strrchr((char *)name,'\\')) == 0)
 		exec_name_ = full_exec_name_;
 	else
 	{
