@@ -1,4 +1,4 @@
-static const char *RcsId = "$Id: pollthread.cpp,v 3.8 2005/07/20 14:40:47 taurel Exp $\n$Name:  $";
+static const char *RcsId = "$Id: pollthread.cpp,v 3.12 2006/05/18 08:51:56 taurel Exp $\n$Name:  $";
 
 //+============================================================================
 //
@@ -13,9 +13,29 @@ static const char *RcsId = "$Id: pollthread.cpp,v 3.8 2005/07/20 14:40:47 taurel
 //
 // author(s) :          E.Taurel
 //
-// $Revision: 3.8 $
+// $Revision: 3.12 $
 //
 // $Log: pollthread.cpp,v $
+// Revision 3.12  2006/05/18 08:51:56  taurel
+// - Miscellaneous changes due to Python device server ported to Windows
+// - Fix some bugs discovered by Windows VC8 using the test suite
+// - Update Windows resource file include path
+// - Fix some Windows VC8 warnings
+//
+// Revision 3.11  2006/04/13 13:31:44  taurel
+// Several changes:
+// - A new DeviceImpl::push_event() method to push USER event with State as data
+// - Fix bug in polling thread POLL_IUPD_PERIOD. The new polling period was applied only after the following poll still done with the old polling period
+// - It is now possible to update polling period of polled objects from a polled attribute or command
+//
+// Revision 3.10  2006/01/27 14:27:10  taurel
+// - Fix a severe incompatibility problem introduced by all the modifs done for
+// PY DS
+// - Duplicate some EventSupplier class methods (instead of using template) in order to be able to generate Tango shared library on Suse 9.3
+//
+// Revision 3.9  2006/01/20 08:22:29  taurel
+// - Added necessary changes to support Device server written in Python
+//
 // Revision 3.8  2005/07/20 14:40:47  taurel
 // - Fix bug in polling thread out of sync error. Time threshold to generate
 // event heartbeat error changed from 10 to 11. Fix some comments
@@ -201,6 +221,7 @@ static const char *RcsId = "$Id: pollthread.cpp,v 3.8 2005/07/20 14:40:47 taurel
 	#include <iomanip.h>
 #endif
 
+extern omni_thread::key_t key_py_data;
 namespace Tango
 {
 
@@ -219,7 +240,7 @@ PollObjType PollThread::type_to_del = Tango::POLL_CMD;
 PollThread::PollThread(PollThCmd &cmd,TangoMonitor &m): shared_cmd(cmd),p_mon(m),
 					    sleep(1),polling_stop(false),
 					    attr_names(1),tune_ctr(1),
-					    need_two_tuning(false)
+					    need_two_tuning(false),auto_upd(-1)
 {
 	attr_names.length(1);
 	
@@ -248,7 +269,11 @@ PollThread::PollThread(PollThCmd &cmd,TangoMonitor &m): shared_cmd(cmd),p_mon(m)
 void *PollThread::run_undetached(void *ptr)
 {
 	PollCmdType received;
+	bool per_thread_data_created = false;
 
+//
+// The infinite loop
+//
 		
 	while(1)
 	{
@@ -259,6 +284,16 @@ void *PollThread::run_undetached(void *ptr)
 			else
 				received = POLL_TIME_OUT;
 
+//
+// Create the per thread data if it is not already done (For Python DS)
+//
+
+			if (per_thread_data_created == false)
+			{
+				omni_thread::self()->set_value(key_py_data,new PyData());
+				per_thread_data_created = true;
+			}
+				
 #ifdef WIN32
 			_ftime(&now_win);
 			now.tv_sec = (unsigned long)now_win.time;
@@ -551,6 +586,21 @@ void PollThread::execute_cmd()
 
 //
 // Update polling period
+// Several cases has to be implemented here.
+// 1 - A classical command from the external world. In this case updating
+//     polling period means removing the already inserted object from the work list,
+//     compute its new polling time and insert it in the work list with its new
+//     polling time and polling period
+// 2 - This is executed by the polling thread itself
+// 2-1 - The command updates polling period for another object: idem than previous
+// 2-2 - The commands updates polling period for the object it is actually polled.
+//       In this case, the object is not in the work list. It has been removed from the
+//	 work list at the beginning of the "one_more_poll" method and is memorized there
+//	 Therefore, simply stores new polling period in a data member. The "one_more_poll"
+//	 method will get its new polling period before re-inserting the object in the work
+// 	 list with the new update period.
+//	 We detect this case because the object is not in any work list (either the work 
+//	 list or the trigger list)
 //
 
 	case Tango::POLL_UPD_PERIOD :
@@ -564,7 +614,35 @@ void PollThread::execute_cmd()
 		if (ite != works.end())
 		{
 			if (local_cmd.new_upd != 0)
-				ite->update = local_cmd.new_upd;
+			{
+				WorkItem tmp_work = *ite;
+#ifdef WIN32
+				unsigned int i,nb_elt;
+				nb_elt = works.size();
+				ite = works.begin();
+				for (i = 0;i < nb_elt;i++)
+				{
+					if (ite->dev == PollThread::dev_to_del)
+					{
+						if (ite->type == PollThread::type_to_del)
+						{
+							if (ite->name == PollThread::name_to_del)
+							{
+								works.erase(ite);
+								break;
+							}
+						}
+					}
+					ite++;
+				}
+#else		
+				works.remove_if(pred);
+#endif
+				tmp_work.update = local_cmd.new_upd;
+				compute_new_date(now,local_cmd.new_upd);
+				tmp_work.wake_up_date = now;
+				insert_in_list(tmp_work);
+			}
 			else
 			{
 			
@@ -614,8 +692,12 @@ void PollThread::execute_cmd()
 // If not found in work list, it should be in the externally
 // triggered object. Therefore, remove it from externally
 // triggered list and insert it in work list
+// If not found in work list and in trig list, we are in case
+// 2-2 as desribed above (polling thread updateing itself polling
+// period of the object it actually polls)
 //
 
+			bool found = false;
 			for (et_ite = ext_trig_works.begin();
 			     et_ite != ext_trig_works.end();++et_ite)
 			{
@@ -626,21 +708,26 @@ void PollThread::execute_cmd()
 						if (et_ite->name == local_cmd.name)
 						{
 							ext_trig_works.erase(et_ite);
+							found = true;
 							break;
 						}
 					}
 				}
 			}
+
+			if (found == true)
+			{			
+				wo.dev = local_cmd.dev;
+				wo.poll_list = &(wo.dev->get_poll_obj_list());
+				wo.type = type_to_del;
+				wo.update = local_cmd.new_upd;
+				wo.name = name_to_del;
+				wo.wake_up_date = now;
 			
-			wo.dev = local_cmd.dev;
-			wo.poll_list = &(wo.dev->get_poll_obj_list());
-			wo.type = type_to_del;
-			wo.update = local_cmd.new_upd;
-			wo.name = name_to_del;
-			wo.wake_up_date = now;
-			
-//			add_random_delay(wo.wake_up_date);
-			insert_in_list(wo);
+				insert_in_list(wo);
+			}
+			else
+				auto_upd = local_cmd.new_upd;
 		}
 		break;
 
@@ -659,7 +746,6 @@ void PollThread::execute_cmd()
 		wo.needed_time.tv_usec = TIME_HEARTBEAT;
 				
 		wo.wake_up_date = now;
-//		add_random_delay(wo.wake_up_date);
 		insert_in_list(wo);
 		break;
 
@@ -723,7 +809,7 @@ void PollThread::execute_cmd()
 		p_mon.signal();
 	}
 	
-	if (Tango::Util::_tracelevel >= 5)
+	if (Tango::Util::_tracelevel >= 4)
 		print_list();
 		
 }
@@ -787,6 +873,22 @@ void PollThread::one_more_poll()
 			break;
 		}
 	}
+
+//
+// For case where the plling thread itself modify the polling
+// period of the object it already polls
+//
+	
+	if (auto_upd != -1)
+	{
+		tmp.update = auto_upd;
+		auto_upd = -1;
+	}
+
+//
+// Compute new polling date and insert work in list
+//
+		
 	compute_new_date(tmp.wake_up_date,tmp.update);	
 	insert_in_list(tmp);
 	tune_ctr--;
@@ -954,9 +1056,8 @@ void PollThread::tune_list(bool from_needed, long min_delta)
 {
 	list<WorkItem>::iterator ite,ite_next,ite_prev;
 	
-	cout4 << "Entering tuning list" << endl;
-		
 	unsigned long nb_works = works.size();
+	cout4 << "Entering tuning list. The list has " << nb_works << " item(s)" << endl;
 
 //
 // Nothing to do if only one let in list
@@ -1143,7 +1244,7 @@ void PollThread::compute_sleep_time()
 
 //+-------------------------------------------------------------------------
 //
-// method : 		PollThread::poll_cmd
+// method : 		PollThread::err_out_of_sync
 // 
 // description : 	To force one event if the polling thread has
 //			discarded one work item because it is late
@@ -1252,7 +1353,6 @@ void PollThread::poll_cmd(WorkItem &to_do)
 		time_diff(before_cmd,after_cmd,needed_time);
 		to_do.needed_time = needed_time;
 #endif
-		after_cmd.tv_sec = after_cmd.tv_sec - DELTA_T;
 	}
 	catch (Tango::DevFailed &e)
 	{
@@ -1459,7 +1559,7 @@ void PollThread::poll_attr(WorkItem &to_do)
 		else
 		{
 			if (idl_vers >= 3)
-				event_supplier->detect_and_push_events(to_do.dev,
+				event_supplier->detect_and_push_events_3(to_do.dev,
 							       	       idl_vers,
 						       	               (*argout_3)[0],
 						                       save_except,
