@@ -1,5 +1,5 @@
-static const char *RcsId = "$Id: utils.cpp,v 3.19 2006/05/25 10:21:23 taurel Exp $\n$Name:  $";
-
+static const char *RcsId = "$Id: utils.cpp,v 3.20 2006/10/06 06:46:53 jensmeyer Exp $\n$Name:  $";
+ 
 //+=============================================================================
 //
 // file :               Tango_utils.cpp
@@ -9,11 +9,14 @@ static const char *RcsId = "$Id: utils.cpp,v 3.19 2006/05/25 10:21:23 taurel Exp
 //
 // project :            TANGO
 //
-// author(s) :          A.Gotz + E.Taurel
+// author(s) :          A.Gotz + E.Taurel 
 //
-// $Revision: 3.19 $
+// $Revision: 3.20 $
 //
 // $Log: utils.cpp,v $
+// Revision 3.20  2006/10/06 06:46:53  jensmeyer
+// Corrected omnithread initialisation in server_init in the case of a windows service.
+//
 // Revision 3.19  2006/05/25 10:21:23  taurel
 // - Create the per-thread data for main thread in server_init() method instead of server_run(). It was the reason of a seg fault when DS aborted in its startup sequence due to device init_device() method throwing exception
 //
@@ -1567,6 +1570,13 @@ void Util::server_init(bool with_window)
 // defined in the binding
 //
 
+#ifdef WIN32
+	if (Util::_service == true)
+	{
+		omni_thread::create_dummy();
+	}
+#endif
+	
 	omni_thread *th = omni_thread::self();
 	if (is_py_ds() == false)
 	{
