@@ -1,4 +1,4 @@
-static const char *RcsId = "$Id: w_attribute.cpp,v 1.8 2005/05/25 06:05:40 taurel Exp $\n$Name:  $";
+static const char *RcsId = "$Id: w_attribute.cpp,v 1.9 2005/11/07 12:15:00 jensmeyer Exp $\n$Name:  $";
 
 //+============================================================================
 //
@@ -14,9 +14,15 @@ static const char *RcsId = "$Id: w_attribute.cpp,v 1.8 2005/05/25 06:05:40 taure
 //
 // author(s) :          E.Taurel
 //
-// $Revision: 1.8 $
+// $Revision: 1.9 $
 //
 // $Log: w_attribute.cpp,v $
+// Revision 1.9  2005/11/07 12:15:00  jensmeyer
+// Corrected initialisation of memorised attributes. No longer exits when receiving an
+// exception during initialisation. Also added an option to only initialise the attribute setpoint and not writing to the attribute.
+// set_memorized_init(false) -> only applies setpoint.
+// set_memorized_init(true) -> also writes setpoint value to the attribute.
+//
 // Revision 1.8  2005/05/25 06:05:40  taurel
 // - Again some minors changes before 5.2
 //
@@ -109,7 +115,7 @@ namespace Tango
 
 WAttribute::WAttribute(vector<AttrProperty> &prop_list,
 		       Attr &tmp_attr,string &dev_name,long idx)
-:Attribute(prop_list,tmp_attr,dev_name,idx),memorized(false),w_ext(NULL)
+:Attribute(prop_list,tmp_attr,dev_name,idx),memorized(false),memorized_init(true),w_ext(NULL)
 {
 
 //
@@ -153,6 +159,8 @@ WAttribute::WAttribute(vector<AttrProperty> &prop_list,
 //
 	
 	set_memorized(tmp_attr.get_memorized());
+	set_memorized_init(tmp_attr.get_memorized_init());
+	
 	if (is_memorized() == true)
 	{
 		try
